# -*- coding: iso-8859-1 -*-

# ==============================================================================
# COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
# ==============================================================================

"""
Tools to maintain a development version of Code_Aster, and useful for
the developper.
Methods are called by an ASTER_RUN object.
"""

__revision__ = "$Id: as_maintenance.py 2670 2006-12-22 13:55:20Z courtois $"


# ----- differ messages translation
def _(mesg):
   return mesg

import sys
import os
import re
import glob
import tarfile
import getpass
from optparse  import SUPPRESS_HELP
from sets      import Set
from types     import ListType, TupleType
EnumTypes = (ListType, TupleType)

from as_run           import RunAsterError
from as_config        import ASTER_CONFIG
from as_build         import ASTER_BUILD
from astk_maintenance import AstkUpdate, get_devel_param

#-------------------------------------------------------------------------------
def SetParser(run):
   """Configure the command-line parser, add options name to store to the list,
   set actions informations.
   run : ASTER_RUN object which manages the execution
   """
   acts_descr = {
      'getop'        : {
         'method' : GetOP,
         'syntax' : '[options] commande[.capy]',
         'help'   : 'return the main subroutine of a Code_Aster command'
      },
      'show'         : {
         'method' : Show,
         'syntax' : '[options] obj1 [obj2...]',
         'help'   : 'show a source file : fortran, C, python, capy, cata, histor or test'
      },
      'get'          : {
         'method' : Get,
         'syntax' : '[options] obj1 [obj2...]',
         'help'   : 'copy a source file in current directory'
      },
      'update'       : {
         'method' : Update,
         'syntax' : '[options] fich1.tar.gz [fich2.tar.gz...]',
         'help'   : 'perform one or several updates of a development version'
      },
      'make'         : {
         'method' : MakeAster,
         'syntax' : '[--version_dev=VERS] [target]',
         'help'   : 'build a Code_Aster version (executable, libraries, catalogues). `target` may be all or clean'
      },
      'auto_update'  : {
         'method' : AutoUpdate,
         'syntax' : '[options]',
         'help'   : 'download available updates from a server and apply them to the current development version'
      },
      'astk_update'  : {
         'method' : AstkUpdate,
         'syntax' : '[options]',
         'help'   : 'download available updates from a server and update astk/as_run itself'
      },
      'getversion'   : {
         'method' : GetVersion,
         'syntax' : '[options]',
         'help'   : 'return current release number of the development version'
      },
      'diag'         : {
         'method' : MakeDiag,
         'syntax' : '[--astest_dir=DIR1,[DIR2]] [--test_list=LIST] [--only_nook]',
         'help'   : 'build the diagnosis of Code_Aster testcases (from DIR or default astest directory)'
      },
      'ctags'         : {
         'method' : GenCtags,
         'syntax' : '[--version_dev=VERS]',
         'help'   : 'build ctags file'
      },
   }
   opts_descr = {
      'local' : {
         'args'   : ('-l', '--local'),
         'kwargs' : {
            'action'  : 'store_true',
            'default' : run['default_local'],
            'dest'    : 'local',
            'help'    : 'files will not been searched on a server but '+ \
                        'on the local machine'
         }
      },
      'nolocal' : {
         'args'   : ('--nolocal', ),
         'kwargs' : {
            'action'  : 'store_true',
            'default' : False,
            'dest'    : 'nolocal',
            'help'    : 'force remote files search (reverse of --local)'
         }
      },
      'vers' : {
         'args'   : ('--vers', ),
         'kwargs' : {
            'type'    : 'string',
            'default' : run['default_vers'],
            'action'  : 'store',
            'dest'    : 'aster_vers',
            'metavar' : 'VERS',
            'help'    : 'Code_Aster version to used (for get, show, getop)'
         }
      },
      'version_dev' : {
         'args'   : ('--version_dev', ),
         'kwargs' : {
            'type'    : 'string',
            'default' : run['version_dev'],
            'action'  : 'store',
            'dest'    : 'version_dev',
            'metavar' : 'VERS',
            'help'    : 'development version of Code_Aster (for make, ' \
                        'update, auto_update, getversion)'
         }
      },
      'all' : {
         'args'   : ('-a', '--all'),
         'kwargs' : {
            'default' : False,
            'action'  : 'store_true',
            'dest'    : 'all_test',
            'help'    : 'get all the files of the test'
         }
      },
      'astest_dir' : {
         'args'   : ('--astest_dir', ),
         'kwargs' : {
            'type'    : 'string',
            'action'  : 'store',
            'dest'    : 'astest_dir',
            'metavar' : 'DIR',
            'help'    : 'testcases directory to watch'
         }
      },
      'only_nook' : {
         'args'   : ('--only_nook', ),
         'kwargs' : {
            'default' : False,
            'action'  : 'store_true',
            'dest'    : 'only_nook',
            'help'    : 'report only errors (but time spent by passed testcases is included)'
         }
      },
      'test_list' : {
         'args'   : ('--test_list', ),
         'kwargs' : {
            'action'  : 'store',
            'dest'    : 'test_list',
            'metavar' : 'LIST',
            'help'    : 'limits the report to these testcases'
         }
      },
      'report_to' : {
         'args'   : ('--report_to', ),
         'kwargs' : {
            'type'    : 'string',
            'default' : '',
            'action'  : 'store',
            'dest'    : 'report_to',
            'metavar' : 'EMAIL',
            'help'    : 'email address to send the report of a execution (only used for --auto_update)'
         }
      },
      'config' : {
         'args'   : ('--config', ),
         'kwargs' : {
            'action'  : 'store',
            'dest'    : 'config',
            'metavar' : 'FILE',
            'help'    : 'use another "config.txt" file (only used for make, updaten auto_update).'
         }
      },
   }
   title = 'Options for maintenance operations'
   run.SetActions(
         actions_descr=acts_descr,
         actions_order=['getop', 'getversion', 'show', 'get', 'make', 'update', \
               'auto_update', 'astk_update', 'diag', 'ctags'],
         group_options=True, group_title=title, actions_group_title=False,
         options_descr=opts_descr
   )

#-------------------------------------------------------------------------------
def GetOP(run, *list_capy):
   """Return the main subroutine of a command.
   """
   REPREF = os.path.join(run.aster_root, run['aster_vers'])
   if len(list_capy) < 1:
      run.parser.error(
            _("'--%s' requires one or more arguments") % run.current_action)
   ficconf = os.path.join(REPREF, 'config.txt')
   conf = ASTER_CONFIG(ficconf, run)
   for capy in list_capy:
      capy = capy.replace('.capy', '')
      nf = os.path.join(REPREF, conf['SRCCAPY'][0], 'commande', capy+'.capy')
      if not os.path.exists(nf):
         run.Mess(_('file not found : %s') % nf, _('<A>_ALARM'))
         run.Mess(_('%s search capy files on local machine') \
               % run.current_action)
         break
      f = open(nf)
      nop = re.search('op *= *([-_0-9a-zA-Z]+)', f.read())
      if nop:
         name = nop.group(1)
         if re.search('[0-9]+', name):
            num = '%04d' % int(name)
            name = 'op'+num+'.f'
         else:
            name = name+'.py'
         Show(run, name)
      else:
         run.Mess(_('op statement not found in %s') % capy, _('<A>_ALARM'))

#-------------------------------------------------------------------------------
def Show(run, *args):
   """Show a source file : fortran, c, python, capy, cata, histor or test.
   """
   kwargs = { 'get' : False }
   Get(run, *args, **kwargs)

#-------------------------------------------------------------------------------
def Get(run, *args, **kwargs):
   """Copy a source file (as Show) in current directory
   """
   if not kwargs.has_key('get'):
      get = True
   else:
      get = kwargs['get']

   if len(args) < 1:
      run.parser.error(
            _("'--%s' requires one or more arguments") % run.current_action)

   copy = True
   if not run['local']:
      user, mach = get_devel_param(run)
      indv = run['aster_vers'][3:4]
      REPREF = os.path.join('/aster', 'v'+indv, run['aster_vers'])
      # répertoires (en distant on suppose une organisation standard)
      RC    = os.path.join(REPREF, 'bibc')
      RFORT = os.path.join(REPREF, 'bibfor')
      RF90  = os.path.join(REPREF, 'bibf90')
      RPY   = os.path.join(REPREF, 'bibpyt')
      RHIST = os.path.join(REPREF, 'histor')
      RCAPY = os.path.join(REPREF, 'catapy')
      RCATA = os.path.join(REPREF, 'catalo')
      RTEST = os.path.join(REPREF, 'astest')
   else:
      user = getpass.getuser()
      mach = os.uname()[1]
      REPREF = os.path.join(run.aster_root, run['aster_vers'])
      # répertoires
      ficconf = os.path.join(REPREF, 'config.txt')
      conf  = ASTER_CONFIG(ficconf, run)
      RC    = os.path.join(REPREF, conf['SRCC'][0])
      RFORT = os.path.join(REPREF, conf['SRCFOR'][0])
      RF90  = os.path.join(REPREF, conf['SRCF90'][0])
      RPY   = os.path.join(REPREF, conf['SRCPY'][0])
      RHIST = os.path.join(REPREF, conf['SRCHIST'][0])
      RCAPY = os.path.join(REPREF, conf['SRCCAPY'][0])
      RCATA = os.path.join(REPREF, conf['SRCCATA'][0])
      RTEST = os.path.join(REPREF, conf['SRCTEST'][0])
      # si 'local' et 'show', pas de copie
      if not get:
         copy = False

   # ----- répertoire temporaire
   if not get:
      rdest = run['cache_dir']
   else:
      rdest = os.getcwd()
   machdest = os.uname()[1]

   fmt = '-- %-44s [%+30s]'

   nberr = 0
   for obj in args:
      toedit = None
      # l'extension fournit le type sauf pour les histor
      ext = obj.split('.')[-1]
      baseobj = re.sub('.%s$' % ext, '', obj)
      if re.search('[0-9]+\.[0-9]+\.[0-9]+', obj):
         ext = 'hist'
      rep = ''
      srep = 0
      ct = 0
      if ext in ('c', 'h'):
         rep = RC
         srep = 1
      elif ext == 'f':
         rep = RFORT
         srep = 1
      elif ext == 'f90':    # but files are suffixed by '.f' !
         ext = 'f'
         obj = re.sub('.f90$', '.f', obj)
         rep = RF90
         srep = 1
      elif ext == 'py':
         rep = RPY
         srep = 1
      elif ext == 'hist':
         rep = RHIST
      elif ext == 'capy':
         rep = RCAPY
         srep = 1
      elif ext == 'cata':
         rep = RCATA
         srep = 1
      elif ext in ('comm', 'mail', 'mess', 'resu', 'para', 'code',
                   'datg', 'msup', 'msh', 'mgib') \
         or re.search('com[0-9]', ext) != None or ext.isdigit():
         rep = RTEST
         ct = 1
      
      if rep == '':
         print fmt % (obj, _('type unsupported'))
         nberr += 1
      else:
         print fmt % (obj, rep)
         jret = 0
         # le fichier existe ou on ne peut pas le vérifier
         if mach != machdest or srep == 1 or \
            (mach == machdest and os.path.isfile(os.path.join(rep, obj))):
         # 1. récupérer le(s) fichier(s)
            if run['all_test'] and ct == 1:
            # cas-test : all
               # est-il dans le cache ?
               if not run['force'] and \
                     os.path.isdir(os.path.join(rdest, baseobj)):
                  print '  |  '+_('from cache')
                  copy = True
               else:
                  src = os.path.join(rep, baseobj + '.*')
                  if copy:
                     if not os.path.isdir(os.path.join(rdest, baseobj)):
                        os.mkdir(os.path.join(rdest, baseobj))
                     if mach == machdest:
                        print '  |  '+_('copy all files of the test')
                     else:
                        print '  |  '+_('remote copy of the test files')
                        src = user+'@'+mach+':'+src
                     jret = run.Copy(os.path.join(rdest, baseobj), src,
                           niverr=_('<E>_COPY_ERROR'))
                  else:
                     toedit = glob.glob(src)
                     print '  |  '+_('just edit')
            else:
            # fichier individuel
               # est-il dans le cache ?
               if not run['force'] and \
                     os.path.isfile(os.path.join(rdest, obj)):
                  print '  |  '+_('from cache')
                  copy = True
               else:
                  if srep == 1:
                     src = os.path.join(rep, '*', obj)
                  else:
                     src = os.path.join(rep, obj)
                  if copy:
                     if mach == machdest:
                        print '  |  '+_('copy')
                     else:
                        print '  |  '+_('remote copy')
                        src = user+'@'+mach+':'+src
                     jret = run.Copy(rdest, src,
                           niverr=_('<E>_COPY_ERROR'))
                  else:
                     toedit = glob.glob(src)
                     print '  |  '+_('just edit')

         # 2. test / edition
            if jret != 0:
               print '  |  '+_('error occurs during copying')
               nberr += 1
            else:
               # si show
               if not get:
                  if not copy:
                     pass
                  elif run['all_test'] and ct == 1:
                  # cas-test : all
                     print '  |  '+_('edit files')
                     toedit = [os.path.join(rdest, baseobj, '*')]
                  else:
                  # fichier individuel
                     print '  |  '+_('edit file')
                     toedit = [os.path.join(rdest, obj)]
                  run.ASSERT(type(toedit) in EnumTypes)
                  if len(toedit) == 0:
                     run.Mess('', _('<E>_FILE_NOT_FOUND'))
                  if type(toedit) in EnumTypes:
                     toedit = ' '.join(toedit)
                  cmd = '%s %s' % (run['editor'], toedit)
                  jret, output = run.Shell(cmd, bg=True)
         else:
         # fichier inexistant
            print ' |   '+_('file not found')
            nberr += 1

   if nberr > 0:
      if nberr > 1:
         s = 's'
      else:
         s = ''
      run.Mess(_('%d error%s detected') % (nberr, s), _('<A>_ALARM'))

#-------------------------------------------------------------------------------
def GetVersion(run, *args, **kwargs):
   """Return release number of current development version :
      result[0:3] : version number
      result[3]   : date of release
      result[4]   : True if "exploitation", False if it's a "development" version
   """
   if len(args) > 0:
      run.parser.error(_("'--%s' requires no argument") % run.current_action)
   # ----- default keywords
   silent = False
   REPREF = os.path.join(run.aster_root, run['version_dev'])
   if kwargs.has_key('silent'):
      silent = kwargs['silent']
   if kwargs.has_key('REPREF'):
      REPREF = kwargs['REPREF']
   
   result = get_version(run, REPREF)
   iret = 0
   if len(result) != 5:
      iret = 4
   elif not silent:
      if result[4]:
         typv = 'exploitation'
      else:
         typv = 'development'
      run.Mess(_('Version %s %s - %s') % (typv, '.'.join(result[:3]), result[3]))
   return iret, result

#-------------------------------------------------------------------------------
def get_version(run, REPREF):
   """Retourne la version du répertoire REPREF.
   """
   ficconf = os.path.join(REPREF, 'config.txt')
   # if file don't exist, suppose default value
   if os.path.exists(ficconf):
      conf = ASTER_CONFIG(ficconf, run)
      bibpyt = conf['SRCPY'][0]
   else:
      bibpyt = 'bibpyt'
   # ----- properties.py
   f = os.path.join(REPREF, bibpyt, 'Accas', 'properties.py')
   if not os.path.isfile(f):
      run.Mess(_('file not found : %s') % f, _('<F>_FILE_NOT_FOUND'))
   mydict = {}
   execfile(f, mydict)
   result = mydict['version'].split('.')
   result.append(mydict['date'])
   result.append(mydict.get('exploit', False))
   return result

#-------------------------------------------------------------------------------
def MakeAster(run, *args):
   """Interface between Makefile and as_run.
   Build Code_Aster from sources, clean object files.
   """
   REPREF = os.path.join(run.aster_root, run['version_dev'])
   if not run.get('config'):
      fconf = os.path.join(REPREF, 'config.txt')
   else:
      fconf = os.path.abspath(run['config'])
   conf = ASTER_CONFIG(fconf, run)

   # check arguments
   target = 'all'
   if len(args) > 1:
      run.parser.error(_("'--%s' takes at most %d argument (%d given)") \
                         % (run.current_action, 1, len(args)))
   elif len(args) > 0:
      target = args[0]

   # set per version environment
   for f in conf.get_with_absolute_path('ENV_SH'):
      run.AddToEnv(f)

   run.print_timer = True

   # 0. ----- working directory during build
   reptrav = os.path.join(run['tmp_user'], 'build.'+run['num_job'])
   run.to_delete.append(reptrav)
   run.MkDir(reptrav)

   # 1. ----- Go !
   if   target == 'all':
      _build_aster(run, conf, False, REPREF, reptrav)
   
   elif target == 'clean':
      l_clean = ['BIN_NODBG', 'BIN_DBG', 'BINCMDE', 'BINELE', 'BINPICKLED',
                 'BINLIB_NODBG', 'BINLIB_DBG', 'BINSHLIB_NODBG', 'BINSHLIB_DBG',
                 'BINLIBF_NODBG', 'BINLIBF_DBG', 'BINSHLIBF_NODBG', 'BINSHLIBF_DBG',
                 'BINOBJ_NODBG', 'BINOBJF_NODBG', 'BINOBJ_DBG', 'BINOBJF_DBG']
      for key in l_clean:
         ficrep = conf[key][0]
         if ficrep != '':
            run.Delete(ficrep, verbose=True)
   
   else:
      run.Mess(_('unknown target : %s') % target, _('<F>_INVALID_ARGUMENT'))

#-------------------------------------------------------------------------------
def _build_aster(run, conf, iupdate, REPREF, reptrav, lardv=None):
   """Build or update Aster depending of iupdate value.
      iupdate : False for 'make', True for 'update'
      lardv   : list of object files to remove from lib_aster
   """
   if lardv == None:
      lardv = []
   build = ASTER_BUILD(run, conf)
   DbgPara = {
      'debug'     : { 'exe'    : conf['BIN_DBG'][0],
                      'suff'   : conf['BINOBJ_DBG'][0],
                      'suffer' : conf['BINOBJF_DBG'][0],
                      'libast' : conf['BINLIB_DBG'][0],
                      'libfer' : conf['BINLIBF_DBG'][0]},
      'nodebug'   : { 'exe'    : conf['BIN_NODBG'][0],
                      'suff'   : conf['BINOBJ_NODBG'][0],
                      'suffer' : conf['BINOBJF_NODBG'][0],
                      'libast' : conf['BINLIB_NODBG'][0],
                      'libfer' : conf['BINLIBF_NODBG'][0]},
   }
   tobuild = conf['MAKE'][0].split()

   # 1a. ----- perform update twice : debug and nodebug modes
   # first, remove object files from libaster
   if iupdate:
      for mode in tobuild:
         libaster = os.path.join(REPREF, DbgPara[mode]['libast'])
         # for backward compatibility
         if run.IsDir(libaster):
            libaster = os.path.join(libaster, 'lib_aster.lib')
         run.Mess(_('Start build in %s mode') % mode, 'TITLE')

         run.timer.Start('Deletion of old files')
         # 'ar -dv' not provided in config.txt
         cmd = [conf['LIB'][0].split()[0], '-dv', libaster]
         cmd.extend(lardv)
         run.VerbStart(_('remove object files from %s') % \
               os.path.basename(libaster), verbose=True)
         if lardv:
            if run['verbose']:
               print
            kret, out = run.Shell(' '.join(cmd))
            # ----- avoid to duplicate output
            if not run['verbose']:
               run.VerbEnd(kret, output=out, verbose=True)
            if kret != 0:
               run.Mess(_('error during deleting objects from archive'),
                     _('<A>_ARCHIVE_ERROR'))
         else:
            run.VerbIgnore(verbose=True)
         run.timer.Stop('Deletion of old files')
         run.CheckOK()
   
   # 1b. ----- perform update twice : debug and nodebug modes
   for mode in tobuild:
      libaster = os.path.join(REPREF, DbgPara[mode]['libast'])
      libferm  = os.path.join(REPREF, DbgPara[mode]['libfer'])
      # for backward compatibility
      if run.IsDir(libaster):
         libaster = os.path.join(libaster, 'lib_aster.lib')
      if run.IsDir(libferm):
         libferm = os.path.join(libaster, 'ferm.lib')
      run.Mess(_('Start build in %s mode') % mode, 'TITLE')      

      # 1.2. compile updated files
      run.timer.Start('Compilation in %s mode' % mode)
      kret = build.CompilAster(REPREF, dbg=mode)
      run.timer.Stop('Compilation in %s mode' % mode)
      run.CheckOK()

      # 1.3. archive '.o'
      # 1.3.1. obj or dbg
      run.timer.Start('Add object files to library')
      kret = build.Archive(repobj=os.path.join(REPREF, DbgPara[mode]['suff']),
            lib=libaster)
      run.CheckOK()

      # 1.3.2. obj_f or dbg_f
      kret = build.Archive(
            repobj=os.path.join(REPREF, DbgPara[mode]['suffer']),
            lib=libferm)
      run.CheckOK()
      run.timer.Stop('Add object files to library')

      # 1.4. update executable
      exec_name = os.path.join(REPREF, DbgPara[mode]['exe'])
      # 1.4.1. build
      run.timer.Start('Build executables')
      kret = build.Link(exec_name, [], libaster, libferm, reptrav)
      run.timer.Stop('Build executables')
      run.CheckOK()
      
      # 1.4.2. testing binary
      # - executable is callable (no shared library unreferenced)
      # - required Python modules are present
      run.timer.Start('Test executables')
      cmd = exec_name + ' -c '
      cpyt=['import traceback']
      cmd_import="""
print '%(cmd)s :',
try:
   %(cmd)s
   print 'OK'
except:
   print 'FAILED'
"""
      cpyt.append(cmd_import % {'cmd' : 'import os'})
      cpyt.append(cmd_import % {'cmd' : 'import Numeric'})
      #cpyt.append(cmd_import % {'cmd' : 'from Numeric import LinearAlgebra'})
      run.VerbStart(_('testing executable %s...') % exec_name, verbose=True)
      if run['verbose']:
         print
      kret, out = run.Shell(cmd + '"' + ''.join(cpyt)+ '"')
      # ----- avoid to duplicate output
      expr = re.compile('(^.*FAILED.*$)', re.MULTILINE)
      l_err = expr.findall(out)
      kret = max(kret, len(l_err))
      if not run['verbose']:
         run.VerbEnd(kret, output=out, verbose=True)
      if kret != 0:
         run.Mess(_('test of executables failed'), _('<F>_BUILD_ERROR'))
      run.timer.Stop('Test executables')

   # 2. ----- compile commands
   run.timer.Start('Compilation of commands catalogue')
   kret = build.CompilCapy(REPREF, reptrav, i18n=True)
   run.timer.Stop('Compilation of commands catalogue')
   run.CheckOK()

   # 3. ----- compile elements
   kargs = {
      'exe'    : os.path.join(REPREF, DbgPara['nodebug']['exe']),
      'cmde'   : os.path.join(REPREF, conf['BINCMDE'][0]),
   }
   run.timer.Start('Make pickled of elements')
   kret = build.MakePickled(REPREF, reptrav)
   run.timer.Stop('Make pickled of elements')
   run.CheckOK()

   run.timer.Start('Compilation of elements')
   kret = build.CompilEle(REPREF, reptrav, **kargs)
   run.timer.Stop('Compilation of elements')
   run.CheckOK()

   # 9. ----- end
   run.Mess(_('Code_Aster has been successfully built'), 'OK')

#-------------------------------------------------------------------------------
def Update(run, *args):
   """Extract "aster-maj" archives and make the update
   """
   REPREF = os.path.join(run.aster_root, run['version_dev'])
   if not run.get('config'):
      fconf = os.path.join(REPREF, 'config.txt')
   else:
      fconf = os.path.abspath(run['config'])
   conf   = ASTER_CONFIG(fconf, run)
   build  = ASTER_BUILD(run, conf)

   # check arguments
   if len(args) < 1:
      run.parser.error(
            _("'--%s' requires one or more arguments") % run.current_action)

   # set per version environment
   for f in conf.get_with_absolute_path('ENV_SH'):
      run.AddToEnv(f)

   run.print_timer = True
   larch = []
   for arch in args:
      larch.append(os.path.abspath(arch))

   # 0. ----- working directory during update(s)
   reptrav       = os.path.join(run['tmp_user'], 'update.'+run['num_job'])
   reptrav_built = os.path.join(run['tmp_user'], 'update_built.'+run['num_job'])
   run.to_delete.append(reptrav)
   run.to_delete.append(reptrav_built)
   run.MkDir(reptrav)
   run.MkDir(reptrav_built)
   prefix = 'maj'
   repmaj = os.path.join(reptrav, prefix)
   run.to_delete.append(repmaj)
   run.MkDir(repmaj)

   run.timer.Start('Extraction of archives')
   # 1. ----- for each archive
   os.chdir(reptrav)
   funig = os.path.join(repmaj, 'unigest')
   lupd  = []
   lunig = []
   for arch in larch:
      arch = os.path.abspath(arch)
      if tarfile.is_tarfile(arch):
         linfo = []

         # 1.1. ----- extraction
         run.VerbStart(_('extract archive %s') % arch, verbose=True)
         jret = 0
         try:
            tar = tarfile.open(arch, 'r')
            tar.errorlevel = 2
            for ti in tar:
               tar.extract(ti)
               # if name is in a previous unigest don't delete it
               name = re.sub('^'+prefix+'/', '', ti.name)
               if os.path.basename(name) != '' and name != 'unigest':
                  lupd.append(name)
               if name in lunig:
                  linfo.append(name)
                  lunig.remove(name)
         except tarfile.ExtractError:
            jret = 4
         run.VerbEnd(jret, verbose=True)
         tar.close()
         if jret != 0:
            run.Mess(_('error during extracting archive %s') % arch,
                  _('<F>_TAR_ERROR'))

         # 1.2. ----- give info about files not to delete
         if linfo:
            run.Mess(_('These files should have previously been deleted ' \
                  'and now they are updated :'))
            print ', '.join(linfo)

         # 1.3. ----- get directives from unigest
         linfo = []
         if os.path.exists(funig):
            for k, val in build.GetUnigest(funig).items():
               if k != 'filename':
                  lunig.extend(val)
            # delete twins
            lunig = dict([(it, 0) for it in lunig]).keys()
            os.remove(funig)
            # delete files from maj which have been modified since last update
            for f in lunig:
               if f in lupd:
                  lupd.remove(f)
               f = os.path.join(repmaj, f)
               if os.path.exists(f):
                  linfo.append(f)

         # 1.4. ----- give info about files to delete before update
         if linfo:
            run.Mess(_('These files have been modified by a previous update ' \
                  'but now they are deleted :'))
            for f in linfo:
               run.Delete(f, verbose=True)

      else:
         run.Mess(_('invalid tar (compressed) archive or ' \
               'file not found : %s') % arch, _('<F>_FILE_NOT_FOUND'))
   run.timer.Stop('Extraction of archives')

   # 2. ----- start update
   # current version
   i, vvv = GetVersion(run, silent=True, REPREF=REPREF)
   if i != 0:
      run.Mess(_('error occurs during getting release number'), _('<F>_ERROR'))
   old_vers = '.'.join(vvv[:3])
   # target version
   i, vnext = GetVersion(run, silent=True, REPREF=repmaj)
   if i != 0:
      run.Mess(_('error occurs during getting release number'), _('<F>_ERROR'))
   new_vers = '.'.join(vnext[:3])
   run.Mess(_('Update version %s to %s') % (old_vers, new_vers), 'TITLE')
   
   # 3. ----- copy files
   run.timer.Start('Copy of files')
   run.Mess(_('Copy updated files'), 'TITLE')
   os.chdir(repmaj)
   ddirs = {
      conf['SRCFOR'][0]  : ['bibfor',   ('*.f', '*.h')],
      conf['SRCF90'][0]  : ['bibf90',   ('*.f', '*.h')],
      conf['SRCC'][0]    : ['bibc',     ('*.c', '*.h')],
      conf['SRCPY'][0]   : ['bibpyt',   '*.py'],
      conf['SRCCAPY'][0] : ['catapy',   '*.capy'],
      conf['SRCCATA'][0] : ['catalo',   '*.cata'],
      conf['SRCFERM'][0] : ['fermetur', '*.f'],
      conf['SRCTEST'][0] : ['test',     '*'],
      conf['SRCHIST'][0] : ['histor',   '*'],
   }
   for repdest, param in ddirs.items():
      if repdest == '':
         run.VerbStart(_('updating %s') % (param[0]), verbose=True)
         run.VerbIgnore(verbose=True)
         continue
      rep  = param[0]
      l_suff = param[1]
      if not type(l_suff) in EnumTypes:
         l_suff = [l_suff,]
      for suff in l_suff:
         if os.path.isdir(rep):
            sub = True
            ldirs = glob.glob(os.path.join(rep, '*/'))
            # if there is no subdirectories (ex : test, fermetur...)
            if len(ldirs) == 0:
               ldirs = [rep, ]
               sub = False
            for entry in ldirs:
               src = os.path.join(entry, suff)
               if not sub:
                  dest = os.path.join(REPREF, repdest)
               else:
                  subdir = os.path.basename(os.path.normpath(entry))
                  dest = os.path.join(REPREF, repdest, subdir)
               run.VerbStart(_('updating %s (%s)') % (dest, suff), verbose=True)
               run.MkDir(dest, verbose=False)
               if len(glob.glob(src)) == 0:
                  run.VerbIgnore(verbose=True)
               else:
                  jret = run.Copy(dest, src)
                  run.VerbEnd(jret, verbose=True)
         elif os.path.isfile(rep):
            dest = os.path.join(REPREF, repdest)
            run.VerbStart(_('updating %s') % dest, verbose=True)
            run.MkDir(dest, verbose=False)
            if rep == 'histor':
               dest = os.path.join(REPREF, repdest, new_vers)
            jret = run.Copy(dest, rep)
            run.VerbEnd(jret, verbose=True)
            
   run.timer.Stop('Copy of files')

   # 4. ----- delete source files
   run.timer.Start('Deletion of old files')
   run.Mess(_('Apply unigest directives'), 'TITLE')
   lardv = []
   for f in lunig:
      run.Delete(os.path.join(REPREF, f), verbose=True)
      if re.search('^'+conf['SRCC'][0]+'/' + \
                  '|^'+conf['SRCFOR'][0]+'/' + \
                  '|^'+conf['SRCF90'][0]+'/', f):
         lardv.append(os.path.basename(re.sub('\.[cf]+$', '.o', f)))
   run.timer.Stop('Deletion of old files')

   # 5. ----- update executable, libs and catalogues
   _build_aster(run, conf, True, REPREF, reptrav_built, lardv=lardv)

#-------------------------------------------------------------------------------
def AutoUpdate(run, *args):
   """Try to download available updates and apply them.
   """
   if len(args) > 0:
      run.parser.error(_("'--%s' requires no argument") % run.current_action)

   run.print_timer = True
   if not run['local']:
      if run['update_protocol'] == 'HTTP':
         REPSRC = run['http_server_user']+'@'+run['http_server_ip']+ \
               ':'+run['http_rep_maj']
      else:
         user, mach = get_devel_param(run)
         REPSRC = '%s@%s:%s' % (user, mach, run['devel_rep_maj'])
   else:
      REPSRC = run['local_rep_maj']

   # version stabilisée courante (ex : 7.2)
   run.timer.Start('Download update archives')
   i, v = GetVersion(run, silent=True, \
         REPREF=os.path.join(run.aster_root, run['version_dev']))
   if i != 0:
      run.Mess(_('error occurs during getting release number'), _('<F>_ERROR'))
   sta   = '%s.%s'    % (v[0], v[1])
   vlast = '%s.%s.%s' % (v[0], v[1], v[2])
   run.Mess(_('Searching updates of %s following release %s...')\
         % (sta, vlast))

   reptrav = os.path.join(run['tmp_user'], 'auto_update.'+run['num_job'])
   run.to_delete.append(reptrav)
   run.MkDir(reptrav)
   os.chdir(reptrav)
   lmaj = []
   # ----- search for sub-releases of current version
   for i in [0, 1]:
      for j in range((1-i)*int(v[2])+1, 50):
         vch = v[0]+'.'+str(int(v[1])+i)+'.'+str(j)
         # nom du fichier archive
         src  = os.path.join(REPSRC, 'aster-maj-src-'+vch+'-1.tar.gz')
         dest = os.path.join(reptrav, os.path.basename(src))
         # use niverr='INFO' to ignore failure
         jret = run.Copy(dest, src, verbose=True, niverr='INFO', \
               protocol=run['update_protocol'])
         if jret != 0:
            if i == 1:
               run.Mess(_('Last release found : %s') % vlast + os.linesep)
            else:
               nextsta = v[0]+'.'+str(int(v[1])+1)
               run.Mess(_('Testing if STA %s was released...') % nextsta)
            break
         else:
            vlast = vch
            lmaj.append(dest)
   run.timer.Stop('Download update archives')

   # ----- start a multiple update
   if lmaj:
      mail = []
      run.ExitOnFatalError = False
      try:
         Update(run, *lmaj)
      except RunAsterError, msg:
         res = 'FAILED'
         run.ExitOnFatalError = True
         import traceback
         mail.extend([_('Exception raised by AutoUpdate:'),
            '-'*60, 'Traceback:'])
         mail.append(''.join(traceback.format_tb(sys.exc_traceback)))
         mail.append('-'*60)
         mail.append(_('Exit code : %s') % msg)
         run.Mess(os.linesep.join(mail), _('<F>_UPDATE_FAILED'))
      else:
         res = 'COMPLETED'
         run.ExitOnFatalError = True
         mail.append(_('Updates ended successfully on %s') % run.GetHostName())
         run.Mess(os.linesep.join(mail), 'OK')
      if run['report_to'] != '':
         run.SendMail(dest=run['report_to'],
            text = os.linesep.join(mail),
            subject = 'AutoUpdate %s %s on %s from %s to %s' % \
                  (run['version_dev'], res, run.GetHostName(), '.'.join(v[:3]),
                   vlast))
   else:
      run.Mess(_('no update found'), 'OK')
   # --- build tags file
   GenCtags(run, force=(len(lmaj) > 0))

#-------------------------------------------------------------------------------
def MakeDiag(run, *args):
   """Build the diagnosis of execution of testcases
   """
   REPREF = os.path.join(run.aster_root, run['aster_vers'])
   fconf = os.path.join(REPREF, 'config.txt')
   conf   = ASTER_CONFIG(fconf, run)
   build  = ASTER_BUILD(run, conf)

   # check arguments
   if len(args) > 0:
      run.parser.error(_("'--%s' requires no argument") % run.current_action)

   run.print_timer = False

   # 0. ----- initializations
   summary = _("""
--- Directory of testcases files    : %s
    Number of test-cases            : %s""")
   txt_vers = _("""    Version                         : %s
""")
   fmt_resu = '%-12s %-18s %10s %10s %10s'
   fmt_tot  = '-'*12 + ' ' + '-'*18 + ' ' + '-'*10 + ' ' + '-'*10 + ' ' + '-'*10
   # 1. ----- Go !
   l_dirs = run.get('astest_dir', os.path.join(REPREF, conf['SRCTEST'][0]))
   l_dirs = [os.path.join(REPREF, os.path.abspath(p.strip())) \
         for p in l_dirs.split(',')]
   # testcases list
   if not run.get('test_list'):
      list_tests = Set()
      for p in l_dirs:
         list_tests.update([os.path.splitext(f)[0] for f in glob.glob1(p, '*.comm')])
      list_tests = list(list_tests)
      list_tests.sort()
   else:
      flist = run['test_list']
      iret, list_tests = get_list(flist)
      if iret != 0:
         run.Mess(_('error during reading file : %s') % flist, _('<F>_ERROR'))
   list_tests.sort()
   nbtest = len(list_tests)
   
   print summary % (', '.join(l_dirs), nbtest)
   lresu  = []
   nbnook = 0
   noresu = 0
   lastv = ''
   i = 0
   for test in list_tests:
      i += 1
      # search the last resu file
      fresu = ''
      ferre = ''
      fmess = ''
      v_i = ''
      for p in l_dirs:
         vtmp = get_vers(os.path.join(p, test + '.resu'), run['verbose'])
         if vtmp > v_i:
            v_i = vtmp
            fresu = os.path.join(p, test + '.resu')
            ferre = os.path.join(p, test + '.erre')
            fmess = os.path.join(p, test + '.mess')
            v_erre = get_vers(ferre, run['verbose'])
            if v_erre != '' and v_erre != v_i:
               ferre = ''
      if lastv == '' or v_i > lastv:
         lastv = v_i
      diag, tcpu, tsys, ttot = build.GetDiag(err=ferre, resu=fresu, mess=fmess, \
                                             cas_test=True)
      lresu.append([os.path.basename(test), diag, tcpu, tsys, ttot, repr_vers(v_i)])
      # count nook
      if run.GetGrav(diag) >= run.GetGrav('NOOK'):
         nbnook += 1
      # count testcase without .resu
      if v_i == '':
         noresu += 1

   t = [0., 0., 0.]
   nbnook = 0
   vlast = repr_vers(lastv)
   print txt_vers % vlast
   for lin in lresu:
      test = lin[0]
      diag = lin[1]
      t[0] += float(lin[2])
      t[1] += float(lin[3])
      t[2] += float(lin[4])
      vers = lin[5]
      # count nook
      if run.GetGrav(diag) >= run.GetGrav('NOOK'):
         nbnook += 1
      if run.GetGrav(diag) >= run.GetGrav('NOOK') \
            or not run['only_nook'] or vers != vlast:
         print fmt_resu % tuple(lin[:-1]),
         if vers != vlast:
            print '%8s' % vers
         else:
            print
   print fmt_tot
   print fmt_resu % (_('%4d tests') % i, _('%d errors') % nbnook, t[0], t[1], t[2])
   print '%4d tests without .resu file' % noresu

#-------------------------------------------------------------------------------
def GenCtags(run, force=False):
   """Generate the ctags file.
   """
   run.Mess(_('Generation of ctags database'), 'TITLE')
   REPREF = os.path.join(run.aster_root, run['version_dev'])
   
   ctags = run.Which('ctags')
   ftags = 'tags'
   if ctags is None:
      run.Mess(_("'ctags' is not in your PATH. ctags not generated."))
      return
   
   style = run.get('ctags_style', '')
   if style == 'exuberant':
      # command line for Exuberant Ctags
      cmd = "find %(rep)s -name '%(suff)s' | %(ctags)s -a -o %(ftags)s -L -"
   elif style == 'emacs':
      # command line for GNU Emacs ctags
      cmd = "find %(rep)s -name '%(suff)s' | %(ctags)s -a -o %(ftags)s -"
   else:
      run.Mess(_("'Exuberant Ctags' and 'GNU Emacs ctags' styles are supported. "\
                 "You must define 'ctags_style' in the config file to generate ctags."))
      return
   
   dcmd = { 'ctags' : ctags, 'ftags' : ftags }
   if not os.access(REPREF, os.W_OK):
      run.Mess(_('no write access to %s') % REPREF, _('<F>_ERROR'))
   
   prev = os.getcwd()
   os.chdir(REPREF)
   
   force = run['force'] or force or (not os.path.isfile(ftags))
   if force:
      run.Delete(ftags, verbose=True)
      for rep, suff in (('bibc', '*.c'),
                        ('bibfor', '*.f'),
                        ('bibf90', '*.f'),
                        ('bibpyt', '*.py'),):
         dcmd['rep']  = rep
         dcmd['suff'] = suff
         run.VerbStart(_('build ctags file using source from %s') % rep, verbose=True)
         if run['verbose']:
            print
         iret, out = run.Shell(cmd % dcmd)
         if not run['verbose']:
            run.VerbEnd(iret, output=out, verbose=True)
         if iret != 0:
            run.Mess(_('error during generating ctags'),_('<E>_CTAGS_ERROR'))
      run.CheckOK()
   else:
      run.Mess(_("ctags file already exists, use --force to overwrite it."))
   
   os.chdir(prev)

#-------------------------------------------------------------------------------
def get_list(ftest):
   """Build list of the tests, ignore all after # or %  (characters comment).
   """
   iret = 0
   ltest = []
   try:
      line = open(ftest, 'r').read().splitlines()
   except (OSError, IOError):
      iret = 4
      return iret, ltest
   
   lfich = [t for t in [re.sub(' *[#%]+.*', '', l).strip() for l in line] if t != '']
   # delete twins (by keeping ordering)
   ltest = []
   for t in lfich:
      if not t in ltest:
         ltest.append(t)
   
   return iret, ltest

#-------------------------------------------------------------------------------
def get_vers(fich, debug=False):
   """Extract the version which produced 'resu'/'erre' file.
   """
   vers = ''
   exp = re.compile('CODE_ASTER.*VERSION.* ([0-9\.]+) ', re.MULTILINE)
   if os.path.isfile(fich):
      l_v = exp.findall(open(fich, 'r').read())
      if len(l_v) > 0:
         vers = l_v[0]
      if debug and vers == '':
         print _('unable to extract version from %s') % fich
   return vers

#-------------------------------------------------------------------------------
def repr_vers(vers, fmt='%d'):
   """Returns a representation of `vers` like '8.3.11' or '08.03.11' using `fmt`.
   """
   if vers == '':
      return vers
   l_v = vers.split('.')
   l_v = l_v + ['0'] * (3 - len(l_v))
   l_out = []
   for i in l_v:
      if not i.isdigit():
         l_out.append(i)
      else:
         l_out.append(fmt % int(i))
   return '.'.join(l_out)




# -*- coding: iso-8859-1 -*-

# ==============================================================================
# COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
# ==============================================================================

"""
Tools to maintain an astk installation.
Note that 'as_run' is a part of astk, so it tries to update itself.

Methods are called by an ASTER_RUN object.
"""

__revision__ = "$Id: astk_maintenance.py 2618 2006-12-15 10:32:40Z courtois $"

# ----- differ messages translation
def _(mesg):
   return mesg

import sys
import os
import re
import tarfile
from glob      import glob

from as_run    import RunAsterError
from utils     import search_cmd, get_absolute_dirname, \
                      IterVersion, change_text_file

#-------------------------------------------------------------------------------
def get_devel_param(run):
   """Returns user/server on the main server.
   """
   user = run['devel_server_user']
   mach = run['devel_server_ip']
   if user == '':
      run.Mess(_("remote connection to '%s' may fail :%s" \
                 "'devel_server_user' not defined in '%s'") \
            % (mach, os.linesep, run.user_rc), _('<A>_ALARM'))
   return user, mach

#-------------------------------------------------------------------------------
def AstkUpdate(run, *args):
   """Update current astk installation.
   """
   if len(args) > 0:
      run.parser.error(_("'--%s' requires no argument") % run.current_action)

   run.print_timer = True
   if not run['local']:
      if run['update_protocol'] == 'HTTP':
         REPSRC = run['http_server_user']+'@'+run['http_server_ip']+ \
               ':'+run['http_rep_maj']
      else:
         user, mach = get_devel_param(run)
         REPSRC = '%s@%s:%s' % (user, mach, run['devel_rep_maj'])
   else:
      REPSRC = run['local_rep_maj']

   # version courante
   if run['version'] == 'VDEV':
      run.Mess(_('It is too dangerous to update a development version !'))
      return
   run.timer.Start('Download update archives')
   run.Mess(_('Searching astk updates following release %s...') % run['version'])

   reptrav = os.path.join(run['tmp_user'], 'astk_update.'+run['num_job'])
   if not run['debug']:
      run.to_delete.append(reptrav)
   run.MkDir(reptrav)
   os.chdir(reptrav)
   
   # format des packages
   fmt_pkg = 'astk-%s-1.noarch.tar.gz'
   fmt_cnt = 'astk-%s'
   
   # ----- search for sub-releases of current version
   itervers = IterVersion(run['version'], 1, 3, 5)
   vlast = run['version']
   fmaj = None
   for vch in itervers:
      # nom du fichier archive
      src  = os.path.join(REPSRC, fmt_pkg % vch)
      dest = os.path.join(reptrav, os.path.basename(src))
      # use niverr='INFO' to ignore failure
      jret = run.Copy(dest, src, niverr='SILENT', \
            protocol=run['update_protocol'])
      if jret == 0:
         vlast = vch
         fmaj = dest
      else:
         itervers.jump()
   
   if vlast != run['version']:
      run.Mess(_('Last release found : %s') % vlast + os.linesep)
   run.timer.Stop('Download update archives')
   
   # ----- start update
   if fmaj:
      mail = []
      run.ExitOnFatalError = False
      try:
         update_astk(run, fmaj, fmt_cnt % vlast, reptrav)
      except RunAsterError, msg:
         res = 'FAILED'
         run.ExitOnFatalError = True
         import traceback
         mail.extend([_('Exception raised by AstkUpdate:'),
            '-'*60, 'Traceback:'])
         mail.append(''.join(traceback.format_tb(sys.exc_traceback)))
         mail.append('-'*60)
         mail.append(_('Exit code : %s') % msg)
         run.Mess(os.linesep.join(mail), _('<F>_UPDATE_FAILED'))
      else:
         res = 'COMPLETED'
         run.ExitOnFatalError = True
         mail.append(_('astk update ended successfully from %s to %s') % \
               (run['version'], vlast))
         run.Mess(os.linesep.join(mail), 'OK')
      if run['report_to'] != '':
         run.SendMail(dest=run['report_to'],
            text = os.linesep.join(mail),
            subject = 'AstkUpdate %s on %s from %s to %s' % \
                  (res, run.GetHostName(), run['version'], vlast))
   else:
      run.Mess(_('no update found'), 'OK')

#-------------------------------------------------------------------------------
def update_astk_dict_trans(run):
   """Retourne le dictionnaire pour les substitution.
   """
   # HOME_ASTK
   home_astk = get_absolute_dirname(run.astk_serv_root)
   
   # récupère le shell utilisé actuellement par as_serv
   fas_serv = os.path.join(run.astk_serv_root, 'bin', 'as_serv')
   shell = open(fas_serv, 'r').readline().strip().replace('#!', '')
   
   # récupère la commande wish utilisée par astk
   fastk = os.path.join(home_astk, 'ASTK_CLIENT', 'bin', 'astk')
   wish_cmd = search_cmd(re.escape('/bin/wish'), fastk)
   
   d_trans = {
      'ASTER_ROOT'      : run.aster_root,
      'SHELL_EXECUTION' : shell,
      'HOME_ASTK'       : home_astk,
      'WISH_EXE'        : wish_cmd or 'wish',
      'TOOLS_DIR'       : os.path.join(run.aster_root, 'outils'),
      'PYTHON_EXE'      : sys.executable,
      'MAXCMDLEN'       : run.system.MaxCmdLen,
   }
   run.DBG(d_trans)
   return d_trans
   
#-------------------------------------------------------------------------------
def update_astk(run, arch, content, reptrav):
   """Réalise la mise à jour d'astk client et serveur à partir du fichier `arch`.
   """
   # 1. ----- extract archive
   os.chdir(reptrav)
   run.timer.Start('Extraction of archives')
   run.Mess(_('Extraction of archives'), 'TITLE')
   if tarfile.is_tarfile(arch):
      # 1.1. ----- extraction
      run.VerbStart(_('extract archive %s') % arch, verbose=True)
      jret = 0
      try:
         tar = tarfile.open(arch, 'r')
         tar.errorlevel = 2
         for ti in tar:
            tar.extract(ti)
      except tarfile.ExtractError:
         jret = 4
      run.VerbEnd(jret, verbose=True)
      tar.close()
      if jret != 0:
         run.Mess(_('error during extracting archive %s') % arch,
               _('<F>_TAR_ERROR'))
   else:
      run.Mess(_('invalid tar (compressed) archive or ' \
            'file not found : %s') % arch, _('<F>_FILE_NOT_FOUND'))
   run.timer.Stop('Extraction of archives')
   if not os.path.isdir(content):
      run.Mess(_("expected a directory '%s' in archive") % content,
            _('<F>_FILE_NOT_FOUND'))

   # 2. ----- configure GUI
   run.timer.Start('Configuration of astk files')
   run.Mess(_('Configuration of astk files'), 'TITLE')
   # --- remplacement des variables
   d_trans = update_astk_dict_trans(run)
   
   repcnt = os.path.join(reptrav, content)
   os.chdir(repcnt)
   l_bin = [os.path.join('ASTK_CLIENT', 'bin', 'astk'),
            os.path.join('ASTK_CLIENT', 'bin', 'bsf'),]
   
   l_bin.extend(glob(os.path.join('ASTK_SERV', 'bin', '*')))
   l_bin.append(os.path.join('ASTK_SERV', 'lib', 'sub_script'))
   
   l_files = l_bin[:]
   l_files.extend(glob(os.path.join('ASTK_CLIENT', 'lib', 'ASTK', 'astkrc', '*')))
   l_files.extend(glob(os.path.join('ASTK_SERV', 'conf', '*')))
   
   run.DBG(l_files)
   extsav = '.astk_update.%s' % run['num_job']
   for ff in l_files:
      if not os.path.isfile(ff):
         run.DBG('not found', ff)
      else:
         run.VerbStart(_('configure %s...') % ff, verbose=True)
         iret = change_text_file(ff, d_trans, delimiter='?', keep=run['debug'], ext=extsav)
         run.VerbEnd(iret, verbose=True)
   
   # --- permissions
   run.DBG(l_bin)
   extsav = '.astk_update.chmod.%s' % run['num_job']
   for ff in l_bin:
      if not os.path.isfile(ff):
         run.DBG('not found', ff)
      else:
         run.VerbStart(_('change permission of %s...') % ff, verbose=True)
         iret = 0
         try:
            os.chmod(ff, 0755)
         except OSError:
            iret = 4
         run.VerbEnd(iret, verbose=True)
   run.timer.Stop('Configuration of astk files')

   # 3. ----- installation
   # --- backup previous configuration
   run.timer.Start('Installation')
   run.Mess(_('Installation'), 'TITLE')
   home_astk = d_trans['HOME_ASTK']
   copy_and_backup(run, home_astk, repcnt,
                   os.path.join('ASTK_CLIENT', 'lib', 'ASTK', 'astkrc'), ext='.new')
   run.Mess(_("""Your current configuration : ASTK_CLIENT/lib/ASTK/astkrc
            new 'astkrc' saved as : ASTK_CLIENT/lib/ASTK/astkrc.new
       See release notes.
"""))
   
   copy_and_backup(run, home_astk, repcnt, os.path.join('ASTK_SERV', 'conf'), ext='.new')
   run.Mess(_("""Your current configuration : ASTK_SERV/conf
            new 'astkrc' saved as : ASTK_SERV/conf.new
       See release notes.
"""))

   # --- real installation
   copy_and_backup(run, repcnt, home_astk, 'ASTK_CLIENT', ext='.old')
   copy_and_backup(run, repcnt, home_astk, 'ASTK_SERV', ext='.old')
   run.timer.Stop('Installation')

#-------------------------------------------------------------------------------
def copy_and_backup(run, src, dest, content, ext=None):
   """Copy content from `src` to `dest`.
   """
   dsrc = os.path.join(src, content)
   ddes = os.path.join(dest, content)
   if os.path.isdir(dsrc):
      if ext:
         dsav = ddes + ext
         if os.path.exists(dsav):
            run.Delete(dsav)
         os.rename(ddes, dsav)
      run.Copy(ddes, dsrc)
   else:
      run.Mess(_("not found : '%s'") % dsrc)




# -*- coding: iso-8859-1 -*-

# ==============================================================================
# COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
# ==============================================================================

"""This class adds an action to the parser.
"""

__revision__ = "$Id: as_option.py 2566 2006-12-08 10:09:20Z courtois $"


from optparse import Option, OptionError, OptionValueError

class ASTER_OPTION(Option):
   """Add 'store_const_once' action, it works like 'store_const' except that
   a value can be stored only once, next occurences will raise an error.
   """
   ACTIONS = Option.ACTIONS + ("store_const_once",)
   STORE_ACTIONS = Option.STORE_ACTIONS + ("store_const_once",)
   TYPED_ACTIONS = Option.TYPED_ACTIONS + ("store_const_once",)

   def take_action (self, action, dest, opt, value, values, parser):
      """Uses 'store_const_once' or standard actions.
      """
      if action == "store_const_once":
         # ----- store_const_once
         if not hasattr(values, dest) or not getattr(values, dest):
            setattr(values, dest, getattr(self, 'const'))
         else:
            raise OptionValueError("%r is invalid because '%s' previously occurs" \
                  % (getattr(self, 'const'), dest))
      else:
         # ----- standard actions
         Option.take_action(self, action, dest, opt, value, values, parser)

   def _check_const (self):
      if self.action != "store_const" and self.action != "store_const_once" \
            and getattr(self, 'const') is not None:
         raise OptionError(
               "'const' must not be supplied for action %r" % self.action, self)

   # ----- because of changes to private method _check_conf
   CHECK_METHODS = [Option._check_action,
                    Option._check_type,
                    Option._check_choice,
                    Option._check_dest,
                    _check_const,
                    Option._check_nargs,
                    Option._check_callback]




# -*- coding: iso-8859-1 -*-

# ==============================================================================
# COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
# ==============================================================================

"""This module encapsulates access to MySQL databases.
"""

__revision__ = "$Id: myconnect.py 2566 2006-12-08 10:09:20Z courtois $"


__all__ = ['CONNECT', 'MySQLError']

# ----- differ messages translation
def _(mesg):
   return mesg

import os
import stat
from types import ListType, TupleType, LongType
EnumTypes = (ListType, TupleType)

import traceback
import MySQLdb

class MySQLError(Exception):
   """Error during MySQLdb commands."""
   pass

class CONNECT:
   """Open a MySQL connection using parameters defined in the file
   `rcdir`/.mysql_connect_`db` where `db` and `rcdir` is given
   through __init__.

   Attributes :
      cursor : Cursor to the active connection.

   Methods :
      exe :   Execute a SQL query
   """
#-------------------------------------------------------------------------------
   def __init__(self, db, **kargs):
      """Open the connection.
      """
      # optionnal arguments
      self.verbose    = kargs.get('verbose', True)
      self.debug      = kargs.get('debug', False)
      self.autocommit = kargs.get('autocommit', True)
      rcdir           = kargs.get('rcdir', os.getenv('HOME'))
      
      fic = os.path.join(rcdir, '.mysql_connect_'+db)
      if not os.path.exists(fic):
         raise IOError, _('No such file or directory %s') % fic

      mydict = dict()
      try:
         execfile(fic, mydict)
      except:
         raise MySQLError, _('Can not read connection parameters')
      try:
         self.connection = MySQLdb.connect(host=mydict['host'], user=mydict['user'],
               passwd=mydict['passwd'], db=mydict['db'])
      except:
         raise MySQLError, _('Connection failed to %s for user %s (and password)') \
                % (mydict['host'], mydict['user'])
      # if autocommit doesn't exist hope it's on by default !
      if self.autocommit and hasattr(self.connection, 'autocommit'):
         self.connection.autocommit(self.autocommit)
      self.cursor = self.connection.cursor()
      if self.verbose:
         print _("""Connection to '%(host)s' on '%(db)s' database.""") % mydict

# ------------------------------------------------------------------------------
   def __del__(self):
      """Close the connection on deletion.
      """
      self.connection.close()

#-------------------------------------------------------------------------------
   def exe(self, query):
      """Execute the SQL 'query' using the active connection.
      Return result lines (as list).
      """
      if self.debug:
         print _('### Query : %s%s--> not executed.') % (query, os.linesep)
         return []
      if not hasattr(self.cursor, 'fetchone'):
         print _('Connection not opened.')
         return []
      try:
         nb = self.cursor.execute(query)
      except:
         traceback.print_exc()
         raise MySQLError, 'ERROR query = %s' % query
      if self.verbose:
         print _('### Query : %s%s--> %d lines returned.') % (query, os.linesep, nb)
      tab = []
      for lig in self.cursor.fetchall():
         tab.append(lig)
      return tab

#-------------------------------------------------------------------------------
   def showtab(self):
      """Return tables of the database (as list).
      """
      res = self.exe("""SHOW TABLES;""")
      li  = []
      for c in res:
         li.append(c[0])
      return li

#-------------------------------------------------------------------------------
   def showcol(self, table):
      """Return names of the columns of the 'table' (as list).
      """
      res = self.exe("""SHOW COLUMNS FROM %s;""" % table)
      li  = []
      for c in res:
         li.append(c[0])
      return li

#-------------------------------------------------------------------------------
   def select_all(self, table):
      """Return all lines of the 'table' (as list).
      """
      return self.exe("""SELECT * FROM %s;""" % table)

#-------------------------------------------------------------------------------
   def insert(self, table, lines):
      """Insert 'lines' (as a list of dict "column:value") into the 'table'.
      Return the number of added lines.
      """
      nb = 0
      if not hasattr(self.cursor, 'fetchone'):
         print _('Connection not opened.')
         return nb
      if not type(lines) in EnumTypes:
         lines = [lines, ]
      for line in lines:
         query = ['INSERT INTO %s SET' % table]
         lset  = []
         for k, v in line.items():
            if v != None:
               if type(v) is LongType:
                  v = '%d' % v
               lset.append('%s=%s' % (k, repr(v)))
         query.append(', '.join(lset))
         query.append(';')
         if self.verbose:
            print _('### Query : %s') % (' '.join(query))
         try:
            nb += self.cursor.execute(' '.join(query))
         except:
            traceback.print_exc()
            raise MySQLError, 'ERROR query = %s' % (' '.join(query))
      if self.verbose:
         print _('--> %d lines added.') % nb
      if nb != len(lines):
         print _('ERROR : not all lines have been added.')
      return nb

#-------------------------------------------------------------------------------
   def update(self, table, line, cond):
      """Update 'line' (as a list of dict "column:value") into the 'table'
      where 'cond' is respected.
      Return the number of updated lines.
      """
      if not hasattr(self.cursor, 'fetchone'):
         print _('Connection not opened.')
         return 0
      query = ['UPDATE %s SET' % table]
      lset  = []
      for k, v in line.items():
         if v != None:
            if type(v) is LongType:
               v = '%d' % v
            lset.append('%s=%s' % (k, repr(v)))
      query.append(', '.join(lset))
      query.append('WHERE')
      lwh = []
      for k, v in cond.items():
         if v != None:
            if type(v) is LongType:
               v = '%d' % v
            lwh.append('%s=%s' % (k, repr(v)))
      query.append(' AND '.join(lwh))
      query.append(';')
      if self.verbose:
         print _('### Query : %s') % (' '.join(query))
      try:
         nb = self.cursor.execute(' '.join(query))
      except:
         traceback.print_exc()
         raise MySQLError, 'ERROR query = %s' % (' '.join(query))
      return nb



# -*- coding: iso-8859-1 -*-
# pylint: disable-msg=W0611

# ==============================================================================
# COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
# ==============================================================================

"""This file is not used by astk server.
It's an example of using an ASTER_RUN object as a toolbox.
(see `as_....` modules to check syntax)
"""
__revision__ = "$Id: alamain.py 2566 2006-12-08 10:09:20Z courtois $"


import os
import sys

ASTER_ROOT = '/opt/aster'

# add .../ASTK_SERV/lib in import path
astk_serv_root = os.path.join(ASTER_ROOT, 'ASTK', 'ASTK_SERV')
sys.path.append(os.path.join(astk_serv_root, 'lib'))

#-------------------------------------------------------------------------------
def ini_run(astk_serv_root):
   """initialization of the ASTER_RUN object
   """
   from as_run       import ASTER_RUN
   from as_system    import ASTER_SYSTEM
   from as_timer     import ASTER_TIMER
   
   run        = ASTER_RUN(astk_serv_root)
   run.options['debug']   = False
   run.options['verbose'] = False
   run.options['force']   = False
   run.options['num_job'] = '000000'
   
   run.timer  = ASTER_TIMER()
   run.system = ASTER_SYSTEM(run, maxcmdlen=32768)
   run.print_timer = True
   iret = 0
   
   return run

#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
if __name__ == '__main__':
   
   VERSION    = 'NEW8'
   REPREF     = os.path.join(ASTER_ROOT, VERSION)
   run = ini_run(astk_serv_root)

   #-------------------------------------------------------------------------------
   # Methods to build Code_Aster
   from as_build     import ASTER_BUILD
   from as_config    import ASTER_CONFIG

   # help(ASTER_BUILD)    prints help on available methods

   fconf = os.path.join(REPREF, 'config.txt')

   conf = ASTER_CONFIG(fconf, run)
   build = ASTER_BUILD(run, conf)

   # Compile all files of a directory
   #build.Compil(typ='C', rep='/opt/aster/NEW8/bibc/supervis', repobj='/tmp/Essai/obj', dbg='nodebug')
   
   # Compile a Code_Aster version
   #build.CompilAster(REPREF, repdest='/tmp/Essai')
   
   # Create a library
   #build.Archive(repobj='/tmp/Essai/obj', lib='/tmp/Essai/mylib.a')
   
   # Build an executable
   #build.Link(exe='/tmp/Essai/aster',
   #           lobj=['/tmp/Essai/mylib.a',],
   #           libaster='/opt/aster/NEW8/lib_obj/libaster.a',
   #           libferm = '/opt/aster/NEW8/lib_obj/libferm.a')

   #-------------------------------------------------------------------------------
   # RunAster function
   import as_exec
   as_exec.SetParser(run)
   # as_exec.RunAster(run, 'my_profile.export')
   
   #-------------------------------------------------------------------------------
   # Utility functions
   import as_maintenance
   as_maintenance.SetParser(run)
   # iret, l_v = as_maintenance.GetVersion(run)
   
   #-------------------------------------------------------------------------------
   # ----- module d'information sur les jobs
   import as_job
   as_job.SetParser(run)
   
   #-------------------------------------------------------------------------------
   # ----- module interface avec le REX
   import as_rex
   as_rex.SetParser(run)
   
   #run.Sortie(iret)



# -*- coding: iso-8859-1 -*-

# ==============================================================================
# COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
# ==============================================================================

"""Definition of ASTER_CONFIG class.
"""

__revision__ = "$Id: as_config.py 2646 2006-12-19 13:31:52Z courtois $"


# ----- differ messages translation
def _(mesg):
   return mesg

import os
import re

from utils     import get_absolute_dirname

#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
class ASTER_CONFIG:
   """Class to read a configuration file of a Code_Aster version ('config.txt')
   and give easy access to the parameters.
   """
#-------------------------------------------------------------------------------
   def __init__(self, config_file, run=None):
      """config_file : filename of the 'config.txt' file to read
      run   : ASTER_RUN object (optional)
      """
      # ----- initialisations
      self.config = {}
      self.filename = config_file
      self.dirn = get_absolute_dirname(config_file)
      self.verbose = False
      self.debug = False
      aster_root = '/opt/aster'

      # ----- reference to ASTER_RUN object which manages the execution
      self.run = run
      if run != None:
         self.verbose = run['verbose']
         self.debug = run['debug']
         aster_root = run.aster_root

      # ----- set optionnal/defaults values (ALWAYS AS LIST !)
      # keep 'Eficas' by default (instead of 'Python') to support older version
      reppy = 'Eficas'
      self.config = {
         # name of source directories
         'SRCFOR'          : ['bibfor'],
         'SRCFERM'         : ['fermetur'],
         'SRCC'            : ['bibc'],
         'SRCPY'           : ['bibpyt'],
         'SRCCATA'         : ['catalo'],
         'SRCCAPY'         : ['catapy'],
         'SRCTEST'         : ['astest'],
         'SRCMAT'          : ['materiau'],
         'SRCHIST'         : ['histor'],
         # name of "binaries" (as results of a make)
         'MAKE'            : ['debug nodebug'],
         'BIN_NODBG'       : ['asteru.exe'],
         'BIN_DBG'         : ['asterd.exe'],
         'BINCMDE'         : ['commande'],
         'BINELE'          : ['elements'],
         'BINPICKLED'      : ['cata_ele.pickled'],
         'BINLIB_NODBG'    : ['lib_obj/lib_aster.lib'], # new values are
         'BINLIB_DBG'      : ['lib_dbg/lib_aster.lib'], #     lib/libaster.a and libasterd.a
         'BINLIBF_NODBG'   : ['lib_obj/ferm.lib'],      # and lib/libferm.a  and libfermd.a
         'BINLIBF_DBG'     : ['lib_dbg/ferm.lib'],
         'BINOBJ_NODBG'    : ['obj'],
         'BINOBJF_NODBG'   : ['obj_f'],
         'BINOBJ_DBG'      : ['dbg'],
         'BINOBJF_DBG'     : ['dbg_f'],
         'BINOBJ_MAIN'     : ['python.o'],
         # name of bibpyt in execution directory
         'REPPY'           : [reppy],
         # modules to compile elements
         'MAKE_SURCH_OFFI' : [os.path.join('Lecture_Cata_Ele', 'make_surch_offi.py')],
         'MAKE_CAPY_OFFI'  : [os.path.join('Lecture_Cata_Ele', 'make_capy_offi.py')],
         # command line arguments for aster executable
         'REPOUT'          : [os.path.join(aster_root, 'outils')],
         'REPMAT'          : [os.path.join(self.dirn, 'materiau')],
         'REPDEX'          : [os.path.join(self.dirn, 'datg')],
         'ARGPYT'          : [os.path.join('Execution', 'E_SUPERV.py')],
         'ARGEXE'          : ['-eficas_path '+reppy],
      }
      
      # ----- read config file
      if not os.path.isfile(config_file):
         self._mess(_('file not found : %s') % config_file, _('<F>_FILE_NOT_FOUND'))

      f = open(config_file, 'r')
      content = f.read()
      f.close()
      self.config.update(self._parse(content))
      
      if self.debug:
         print '<DBG> <init> ASTER_CONFIG :'
         print self

#-------------------------------------------------------------------------------
   def __repr__(self):
      """Pretty print configuration
      """
      fmt = ' %-10s = %s'
      txt = []
      txt.append(fmt % ('Filename', self.filename))
      sorted_keys = self.config.keys()
      sorted_keys.sort()
      for key in sorted_keys:
         txt.append(fmt % (key, self.config[key]))
      return os.linesep.join(txt)

#-------------------------------------------------------------------------------
   def __getitem__(self, key):
      """Return the value of parameter 'key', or '' if not exists
      (so never raise KeyError exception).
      """
      if self.config.has_key(key):
         return self.config[key]
      else:
         return ['']

#-------------------------------------------------------------------------------
   def get_with_absolute_path(self, key, sep=' '):
      """For fields containing pathnames returns absolute path names.
      """
      l_res = []
      for val in self[key]:
         l_res.extend([os.path.join(self.dirn, path) for path in val.split()])
      return l_res

#-------------------------------------------------------------------------------
   def _mess(self, msg, cod='', store=False):
      """Just print a message
      """
      if hasattr(self.run, 'Mess'):
         self.run.Mess(msg, cod, store)
      else:
         print '%-18s %s' % (cod, msg)

#-------------------------------------------------------------------------------
   def _parse(self, content):
      """Extract fields of config from 'content'
      """
      cfg = {}
      self.content = content
      for l in content.split(os.linesep):
         if not re.search('^[ ]*#', l):
            try:
               typ, nam, ver, val = l.split('|')
               #print '========>', typ, '//', nam, '//', ver, '//', val
               typ = re.sub('^[ ]*', '', re.sub('[ ]*$', '', typ))
               val = re.sub('^[ ]*', '', re.sub('[ ]*$', '', val))
               if val != '':
                  if cfg.has_key(typ):
                     cfg[typ].append(val)
                  else:
                     cfg[typ] = [val]
            except ValueError:
               pass
      return cfg




# -*- coding: iso-8859-1 -*-

# ==============================================================================
# COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
# ==============================================================================

"""Definition of ASTER_PROFIL class.
"""

__revision__ = "$Id: as_profil.py 2566 2006-12-08 10:09:20Z courtois $"


# ----- differ messages translation
def _(mesg):
   return mesg

import os
import re
from types import ListType, TupleType
EnumTypes = (ListType, TupleType)

#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
class ASTER_PROFIL:
   """Class to read and parse an exported ASTK profile.
   Attributes :
      filename : absolute filename of the '.export' file
      param    : value of each parameters (type dict) (each param is a list)
      args     : arguments (type dict)
      data     : list of the files/directories which are datas
      resu     : list of the files/directories which are results
         (each of data/resu is a dict of : path, ul, compr, type, rep)
   """
#-------------------------------------------------------------------------------
   def __init__(self, filename=None, run=None):
      """filename : filename of the '.export' file to read
      run   : ASTER_RUN object (optional)
      """
      # initialisations
      self.param = {}
      self.args = {}
      self.data = []
      self.resu = []
      self.filename = filename
      self.verbose = False
      self.debug = False
      # ----- reference to ASTER_RUN object which manages the execution
      self.run = run
      if run != None:
         self.verbose = run['verbose']
         self.debug   = run['debug']
      if filename != None:
         # check if file exists
         if not os.path.isfile(filename):
            self._mess(_('file not found : %s') % filename, _('<A>_ALARM'))
         else:
            # read the file
            f = open(filename, 'r')
            content = f.read()
            f.close()
            self._parse(content)
      
      if self.debug:
         print '<DBG> <init> ASTER_PROFIL :'
         print self

#-------------------------------------------------------------------------------
   def __repr__(self):
      """Pretty print content of the profile
      """
      fmt = ' %-10s = %s'
      fmt2 = ' %-6s %2s %1s %-s %1s'
      txt = []
      txt.append(fmt % ('Filename', self.filename))
      for comm, d in ('Parameters :', self.param), ('Arguments :', self.args):
         txt.append(comm)
         sorted_keys = d.keys()
         sorted_keys.sort()
         for key in sorted_keys:
            txt.append(fmt % (key, d[key]))
      for comm, l in ('Data :', self.data), ('Result :', self.resu):
         txt.append(comm)
         for d in l:
            c = ' '
            if d['compr']:
               c = 'C'
            rep = ' '
            if d['isrep']:
               rep = 'R'
            txt.append(fmt2 % (d['type'], d['ul'], c, d['path'], rep))
      return os.linesep.join(txt)

#-------------------------------------------------------------------------------
   def __getitem__(self, key):
      """Return the value of parameter 'key', or '' if not exists
      (so never raise KeyError exception).
      """
      if self.param.has_key(key):
         return self.param[key]
      else:
         return ['']

#-------------------------------------------------------------------------------
   def Get(self, DR, typ):
      """Return the list of datas (DR='D') or results (DR='R') or both (DR='DR')
      of the type 'typ'
      """
      lr = []
      if   DR == 'D':
         what = self.data
      elif DR == 'R':
         what = self.resu
      else:
         what = self.data + self.resu
      for entry in what:
         if entry['type'] == typ:
            lr.append(entry)
      return lr

#-------------------------------------------------------------------------------
   def Set(self, DR, dico):
      """Add an entry defined in `dico` in .data/.resu.
      """
      if   DR == 'D':
         l_what = [self.data]
      elif DR == 'R':
         l_what = [self.resu]
      else:
         l_what = [self.data, self.resu]
      l_k = ('type', 'isrep', 'path', 'ul', 'compr')
      for k in l_k:
         if not dico.has_key(k):
            self._mess(_("key '%s' missing") % k, _('<F>_PROGRAM_ERROR'))
      for what in l_what:
         what.append(dico)

#-------------------------------------------------------------------------------
   def __setitem__(self, p, v):
      """Add the parameter 'p' with the value 'v'
      """
      if self.debug and self.param.has_key(p):
         print '<DBG> (AS_PROFIL.setitem) force param['+p+'] = ', v
      if type(v) not in EnumTypes:
         v = [v,]
      self.param[p] = v

#-------------------------------------------------------------------------------
   def _mess(self, msg, cod='', store=False):
      """Just print a message
      """
      if hasattr(self.run, 'Mess'):
         self.run.Mess(msg, cod, store)
      else:
         print '%-18s %s' % (cod, msg)

#-------------------------------------------------------------------------------
   def _parse(self, content):
      """Extract fields of config from 'content'
      """
      self.content = content
      for l in content.split(os.linesep):
         if not re.search('^[ ]*#', l):
            spl = l.split()
            typ = ''
            if len(spl) > 0:
               typ = spl[0]
               if typ not in ('A', 'P', 'F', 'R'):
                  self._mess(_('unexpected type : %s') % typ, _('<A>_ALARM'))
            else:
               continue
            if len(spl) >= 3:
               if typ == 'P':
                  if self.param.has_key(spl[1]):
                     self.param[spl[1]].append(' '.join(spl[2:]))
                  else:
                     self.param[spl[1]] = [' '.join(spl[2:])]
               elif typ == 'A':
                  self.args[spl[1]] = ' '.join(spl[2:])
               elif typ == 'F' or typ == 'R':
                  if len(spl) >= 5:
                     dico = {
                        'type'  : spl[1],
                        'path'  : spl[2],
                        'isrep' : typ == 'R',
                        'ul'    : spl[4],
                        'compr' : spl[3].find('C') > -1,
                     }
                     if spl[3].find('D')>-1:
                        self.data.append(dico)
                     if spl[3].find('R')>-1:
                        self.resu.append(dico)
                  else:
                     self._mess(_('fields missing on line : %s') % l, _('<A>_ALARM'))
            elif len(spl) >= 2:
               typ = spl[0]
               if typ == 'A':
                  self.args[spl[1]] = ''

#-------------------------------------------------------------------------------
   def copy(self):
      """Return a copy of the profile
      """
      newp = self.__class__(None, self.run)
      for attr in ('args', 'param', 'data', 'resu'):
         setattr(newp, attr, getattr(self, attr))
      return newp
      
#-------------------------------------------------------------------------------
   def WriteExportTo(self, fich):
      """Write the export file represents this profile.
      """
      self.filename = self.filename or fich
      txt = []
      for p, dico in ('P', self.param), ('A', self.args):
         sorted_keys = dico.keys()
         sorted_keys.sort()
         for key in sorted_keys:
            l_val = dico[key]
            if type(l_val) not in EnumTypes:
               l_val = [l_val,]
            for v in l_val:
               txt.append(' '.join([p, key, str(v)]))
      for dr, l_val in ('D', self.data), ('R', self.resu):
         for d in l_val:
            c = ' '
            if d['compr']:
               c = 'C'
            fr = 'F'
            if d['isrep']:
               fr = 'R'
            txt.append(' '.join([fr, d['type'], d['path'], dr+c, str(d['ul'])]))
      txt.append('')
      try:
         open(fich, 'w').write(os.linesep.join(txt))
      except IOError, msg:
         self._mess(_('No write access to %s') % fich, _('<F>_ERROR'))





# -*- coding: iso-8859-1 -*-

# ==============================================================================
# COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
# ==============================================================================

"""
Define classes representing an histor file from issues objects.
"""

__revision__ = "$Id: histor.py 2626 2006-12-18 07:50:27Z courtois $"


# ----- differ messages translation
def _(mesg):
   return mesg

import os
import time
import datetime
import xml.sax.saxutils as saxutils
from types import StringType, FloatType, IntType, LongType, ListType, TupleType

import mystring

NumberTypes = (FloatType, IntType, LongType)
EnumTypes = (ListType, TupleType)

french_date_fmt = '%d/%m/%Y'

class HistorError(Exception):
   """Exception for this module
   """

#-------------------------------------------------------------------------------
def date_inverse(date, sep='/'):
   """Reverse the representation of a date : dd/mm/YY <---> YY/mm/dd.
   """
   if isinstance(date, datetime.datetime) or type(date).__name__ == 'DateTime':
      return date.strftime(french_date_fmt)
   else:
      # suppose `date` is in mysql format
      l_d = str(date).split(sep)
      l_d.reverse()
      return sep.join(l_d)

#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
class HISTOR:
   """Class representing a "standard" histor i.e. as a text file.
   """
   CR = os.linesep
   format = 'text'
   header = ''
   footer = ''
   separ = '-'*80+CR
   seprep = CR*2
   mask = separ + """RESTITUTION FICHE %(id)06d DU %(date_emission)s
TYPE %(type_name)s concernant %(produit_name)s (VERSION %(version_name)s)
%(tma)sTITRE
   %(_title)s
FONCTIONNALITE
%(reponse)s
RESU_FAUX_VERSION_EXPLOITATION    :  %(_fauxVexpl)s
RESU_FAUX_VERSION_DEVELOPPEMENT   :  %(_fauxVdev)s
RESTITUTION_VERSION_EXPLOITATION  :  %(_corrVexpl)s
RESTITUTION_VERSION_DEVELOPPEMENT :  %(_corrVdev)s
IMPACT_DOCUMENTAIRE : %(_impactDoc)s
VALIDATION
   %(_validation)s
%(nbjours)s"""
   mask_nbjours = "NB_JOURS_TRAV  : %s" + CR
   mask_tma = 'TMA : %s' + CR
   mask_user_begin = separ + \
    '--- AUTEUR %(_loginaster)-12s %(_realname)-20s   DATE %(dateheure)s' + CR*2
   mask_user_end = separ + CR

#-------------------------------------------------------------------------------
   def __init__(self, **kargs):
      """Initialize histor object.
      """
      self.l_bloc = []

#-------------------------------------------------------------------------------
   def clean_dict(self, dico):
      """Keep only elements of dict of type string and number,
      and escape all HTML characters.
      """
      newd = {}
      for k, v in dico.items():
         if k == 'CR' or type(v) is StringType or type(v) in NumberTypes:
            newd[k] = v
         elif v is None:
            newd[k] = ''
      return newd

#-------------------------------------------------------------------------------
   def DictIssue(self, issue, l_rep=None):
      """Represent an issue as a dict object.
      l_rep : list of the content of messages to include.
      """
      # 0. store issue infos
      dico = issue.values.copy()
      # _version is not always defined
      try:
         dico['version_name'] = issue['_version'].GetPrimValue()
      except AttributeError:
         dico['version_name'] = ''
      dico.update({
         'CR'            : self.CR,
         'type_name'     : issue['_type'].GetPrimValue(),
         'produit_name'  : issue['_produit'].GetPrimValue(),
         'reponse'       : '',
         'tma'           : '',
         'nbjours'       : '',
         'date_emission' : date_inverse(issue['_creation']),
      })
      
      # 1. list of messages or last message
      if l_rep != None and len(l_rep) > 0:
         if not type(l_rep) in EnumTypes:
            l_rep = [l_rep]
         l_rep = [mystring.indent(s, '   ') for s in l_rep]
         dico['reponse'] = self.seprep.join(l_rep)
      else:
         d_msg = issue.GetLinks()
         imsg = max(d_msg.keys())
         dico['reponse'] = d_msg[imsg]['_summary'] + '...'
      
      # 2. champs OUI/NON
      for key in ('_corrVdev', '_corrVexpl'):
         if dico[key] != None:
            dico[key] = 'OUI'
         else:
            dico[key] = 'NON'
      for key in ('_fauxVdev', '_fauxVexpl'):
         if dico[key] != None:
            dico[key] = 'OUI   DEPUIS : %s' % dico[key]
         else:
            dico[key] = 'NON'
      
      # 3. "pour le compte de" field
      if issue['_intervenant'] != None:
         dico['tma'] = self.mask_tma % issue['_intervenant'].GetPrimValue()
      
      # 4. show '_nbJours' field if no intervenant
      if issue['_intervenant'] == None:
         dico['nbjours'] = self.mask_nbjours % issue['_nbJours']
      
      # 5. keep only strings
      dico = self.clean_dict(dico)
      
      return dico

#-------------------------------------------------------------------------------
   def AddIssue(self, issue, l_rep=None):
      """Add a bloc for an issue.
      """
      self.l_bloc.append(self.mask % self.DictIssue(issue, l_rep))

#-------------------------------------------------------------------------------
   def DictUser(self, user):
      """Represent an user as a dict object.
      """
      dico = self.clean_dict(user.values.copy())
      
      dico.update({
         'CR'            : self.CR,
         'dateheure'     : time.strftime('%d/%m/%Y - %H:%M:%S'),
      })
      return dico

#-------------------------------------------------------------------------------
   def BeginUser(self, user):
      """Start a bloc to identify the user who answers following issues.
      """
      self.l_bloc.append(self.mask_user_begin % self.DictUser(user))

#-------------------------------------------------------------------------------
   def EndUser(self, user):
      """End the bloc.
      """
      self.l_bloc.append(self.mask_user_end % self.DictUser(user))

#-------------------------------------------------------------------------------
   def __repr__(self):
      """Modify representation.
      """
      return self._getrepr()

#-------------------------------------------------------------------------------
   def _getrepr(self):
      """Representation of the histor object.
      """
      txt = [self.header]
      txt.extend(self.l_bloc)
      txt.append(self.footer)
      return ''.join(txt)

#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
class HISTOR_HTML(HISTOR):
   """Class representing a histor in HTML format.
   """
   CR = '<br />'
   format = 'html'
   header = """<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<html>
<head>
   <meta http-equiv="content-type" content="text/html; charset=ISO-8859-1">
   <title>histor - Code_Aster</title>
</head>
   <style type="text/css">
      body {
         font-family: monospace;
         text-align: justify;
      }
      .fiche {
         font-style: normal;
         line-height: 1.5em ;
      }
      .par {
         margin-left: 50px ;
         line-height: 1.3em ;
      }
      .enteteuser {
         font-weight: bold;
         text-align: center;
      }
   </style>
<body>
"""
   footer = """</body>
</html>
"""
   separ = '<hr />'
   mask = separ + """

<!-- @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ -->
<div class="fiche">
RESTITUTION FICHE <a href="%(url_issue)s">%(id)06d</a> DU %(date_emission)s%(CR)s
TYPE %(type_name)s concernant %(produit_name)s (VERSION %(version_name)s)%(CR)s
%(tma)s%(CR)sTITRE
<div class="par">
   %(_title)s
</div>
FONCTIONNALITE
<div class="par">
   %(reponse)s
</div>
RESU_FAUX_VERSION_EXPLOITATION&nbsp;&nbsp;&nbsp;&nbsp;:  %(_fauxVexpl)s%(CR)s
RESU_FAUX_VERSION_DEVELOPPEMENT&nbsp;&nbsp;&nbsp;:  %(_fauxVdev)s%(CR)s
RESTITUTION_VERSION_EXPLOITATION&nbsp;&nbsp;:  %(_corrVexpl)s%(CR)s
RESTITUTION_VERSION_DEVELOPPEMENT&nbsp;:  %(_corrVdev)s%(CR)s
IMPACT_DOCUMENTAIRE : %(_impactDoc)s%(CR)s
VALIDATION
<div class="par">
   %(_validation)s
</div>
%(nbjours)s%(CR)s</div>
"""
   mask_user_begin = """
<!-- @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ -->
%(CR)s
<p class="enteteuser">
   AUTEUR %(_loginaster)s %(_realname)s &nbsp;&nbsp;&nbsp;&nbsp;
   DATE %(dateheure)s
</p>
"""
   mask_nbjours = "NB_JOURS_TRAV  : %s"
   mask_tma = 'TMA : %s'
   mask_user_end = separ + CR

#-------------------------------------------------------------------------------
   def __init__(self, **kargs):
      """Initialize histor object.
      """
      HISTOR.__init__(self)
      
      # specific arguments
      self.server_url = kargs.get('url', None)
      self.fmt_url_issue = '%s/issue%%s' % saxutils.escape(self.server_url)

#-------------------------------------------------------------------------------
   def clean_dict(self, dico):
      """Keep only elements of dict of type string and number,
      and escape all HTML characters.
      """
      newd = {}
      for k, v in dico.items():
         if k == 'CR' or type(v) in NumberTypes:
            newd[k] = v
         elif type(v) is StringType:
            newd[k] = saxutils.escape(str(v))
         elif v is None:
            newd[k] = ''
      return newd

#-------------------------------------------------------------------------------
   def DictIssue(self, issue, l_rep=None):
      """Represent an issue as a dict object.
      """
      dico = HISTOR.DictIssue(self, issue, l_rep)
      
      # specific fields
      dico['url_issue'] = self.fmt_url_issue % issue['id']
      
      # multi-lines fields
      for k in ('reponse',):
         dico[k] = dico[k].replace(os.linesep, self.CR)
      
      return dico


#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
def InitHistor(format='text', **kargs):
   """Return a histor object according to `format`.
   """
   if format == 'text':
      return HISTOR(**kargs)
   elif format == 'html':
      return HISTOR_HTML(**kargs)
   else:
      raise HistorError, _('Unknown histor format : %s') % str(format)




# -*- coding: iso-8859-1 -*-

# ==============================================================================
# COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
# ==============================================================================

"""
Give methods to :
- create a new issue in the tracker database,
- ...
Methods are called by an ASTER_RUN object.
"""

__revision__ = "$Id: as_rex.py 2626 2006-12-18 07:50:27Z courtois $"


# ----- differ messages translation
def _(mesg):
   return mesg

import os
import os.path
import re
import time
import getpass

from mystring import convert
import as_maintenance

try:   
   import grp
   import myconnect as db
   from schema    import ReadDB, TYPE, USER, PRODUIT, STATUS, ISSUE, MSG, \
                         ISSUE_MESSAGES, YES, NO, ReadDBError, WriteDBError
   from histor    import InitHistor
   imports_succeed = True
except ImportError:
   imports_succeed = False

std_encoding = 'iso-8859-1'
db_encoding  = 'utf-8'

#-------------------------------------------------------------------------------
def SetParser(run):
   """Configure the command-line parser, add options name to store to the list,
   set actions informations.
   run : ASTER_RUN object which manages the execution
   """
   acts_descr = {
      'create_issue' : {
         'method' : Creer,
         'syntax' : 'issue_file [export_file]',
         'help'   : 'insert a new issue in database and copy attached files '\
                    'if an export file is provided.'
      },
      'extract_histor' : {
         'method' : Histor,
         'syntax' : '[--status=STAT] [--format=FORM] [--all_msg] input_file histor',
         'help'   : 'extract the content of issues listed in `input_file` to `histor`.'
      },
      'close_issue' : {
         'method' : Solder,
         'syntax' : '--version_dev=VERS histor',
         'help'   : 'fill "corrVdev" or "corrVexpl" field (depends on VERS) in issues '\
                    'found in `histor` and eventually close them.'
      },
   }
   opts_descr = {
      'status' : {
         'args'   : ('--status', ),
         'kwargs' : {
            'action'  : 'store',
            'default' : 'all',
            'metavar' : 'STAT',
            'choices' : ('all', 'resolu', 'valide_EDA', 'ferme'),
            'dest'    : 'status',
            'help'    : 'raise an error if issues are not in this status'
         }
      },
      'format' : {
         'args'   : ('--format', ),
         'kwargs' : {
            'action'  : 'store',
            'default' : 'text',
            'metavar' : 'FORM',
            'choices' : ('text', 'html'),
            'dest'    : 'format',
            'help'    : 'format of generated histor file'
         }
      },
      'all_msg' : {
         'args'   : ('--all_msg', ),
         'kwargs' : {
            'action'  : 'store_true',
            'default' : False,
            'dest'    : 'all_msg',
            'help'    : 'retrieve all the messages of issues'
         }
      },
   }
   title = 'Options for issue tracker interface'
   run.SetActions(
         actions_descr=acts_descr,
         actions_order=['create_issue', 'extract_histor', 'close_issue'],
         group_options=True, group_title=title, actions_group_title=False,
         options_descr=opts_descr,
   )

#-------------------------------------------------------------------------------
def Creer(run, *args):
   """Create a new issue in REX database.
   """
   if len(args) < 1 or len(args) > 2:
      run.parser.error(_("'--%s' requires one or two arguments") % run.current_action)

   iret = 0
   on_machref = run.get('rep_agla', 'local') != 'local'
   if not on_machref:
      run.Mess(_('Only available on the AGLA machine'), _('<F>_AGLA_ERROR'))
   
   # 0. check imports
   if not imports_succeed:
      run.Mess(_('Imports of REX interface failed.'), _('<F>_IMPORT_ERROR'))

   # 1. copy export and issue files
   jn = run['num_job']
   ffich = os.path.join(run['tmp_user'], 'rex_fich.'+jn)
   kret = run.Copy(ffich, args[0], niverr=_('<F>_COPYFILE'))
   fichetude = 0
   if len(args) > 1:
      fichetude = 1
      fprof = os.path.join(run['tmp_user'], 'rex_prof.'+jn)
      kret = run.Copy(fprof, args[1], niverr=_('<F>_PROFILE_COPY'))

   # 2. parse issue file
   fiche = open(ffich, 'r').read()
   l_champs = ['NOMUSER', 'MAILUSER', 'UNIUSER',
               'TITRE', 'DATE1', 'DATE2', 'VERSION', 'TYPFIC', 'TEXTE',
               'FICASS']
   d = {}
   for ch in l_champs:
      exp = re.compile('\@%s\@(.*)\@FIN%s\@' % (ch, ch), re.MULTILINE | re.DOTALL)
      mat = exp.search(fiche)
      if mat != None and mat.group(1).strip() != 'Non_défini':
         d[ch] = mat.group(1).strip()
   if run['debug']:
      print 'Dict issue content : ', repr(d)

   # 3. open database connection
   try:
      c = db.CONNECT('REX',
            rcdir = os.path.join(run.astk_serv_root, 'conf'), verbose=run['debug'])
   except Exception, msg:
      run.Mess(msg, _('<F>_DB_ERROR'))
      
   typ = ReadDB(TYPE, c)
   d_typ = {
      'AL'   : typ['anomalie'],
      'EL'   : typ['evolution'],
      'AOM'  : typ['aide utilisation'],
      'AO'   : typ['anomalie'],
      'EO'   : typ['evolution'],
      'ED'   : typ['evolution'],
   }
   if not d['TYPFIC'] in d_typ.keys():
      run.Mess(_('Unknown issue type : %s') % d['TYPFIC'], _('<F>_PARSE_ERROR'))

   # 4. create new issue
   # 4.1. get fields from db
   login = getpass.getuser()
   loginrex = login
   try:
      res = c.exe("""SELECT id, _username FROM _user WHERE _loginaster='%s';""" % login)
      loginrex = res[0][1]
      if len(res) > 1:
         run.Mess(_("More than one user has '%s' as login in REX database, " \
                    "'%s' is taken") % (login, loginrex),
            _('<A>_MULTIPLE_USER'))
   except (IndexError, db.MySQLError), msg:
      run.Mess(str(msg))
      run.Mess(_('User %s unknown in REX database') % login,
            _('<F>_UNKNOWN_USER'))
   user = USER({'_username' : loginrex}, c)
   try:
      user.read()
   except ReadDBError:
      run.Mess(_('User %s unknown in REX database') % login,
            _('<F>_UNKNOWN_USER'))
   prod = PRODUIT({'_name' : 'Code_Aster'}, c)
   try:
      prod.read()
   except ReadDBError:
      run.Mess(_('Code_Aster product not found in database !'), _('<F>_DB_ERROR'))
   emis = STATUS({'_name' : 'emis'}, c)
   try:
      emis.read()
   except ReadDBError:
      run.Mess(_("Status 'emis' not found in database !"), _('<F>_DB_ERROR'))

   # 4.2. get version item
   d_vers = prod.GetLinks()
   vers = d_vers.get(d['VERSION'])
   if vers == None:
      run.Mess(_("Version %s not found in database !") % d['VERSION'])

   # 4.3. fill fields
   date_now = time.strftime('%Y/%m/%d')
   issue = ISSUE({
         '_creator'  : user,
         '_produit'  : prod,
         '_status'   : emis,
         '_title'    : d['TITRE'],
         '_type'     : d_typ[d['TYPFIC']],
         '_version'  : vers,
         '_fichetude' : fichetude,
      }, c)
   descr = MSG({'_author'   : user,
                '_creation' : date_now,
                '_creator'  : user,
                '_date'     : date_now,
                '_summary'  : d['TEXTE'][:255], } ,c)
   lien = ISSUE_MESSAGES({'linkid' : descr, 'nodeid' : issue}, c)
   
   # 4.4. insert issue in database
   try:
      lien.write()
   except WriteDBError, msg:
      run.Mess(_('Insert issue failed'), _('<F>_DB_ERROR'))
   
   # 5. get message and issue id
   numid = issue['id']
   msgid = descr['id']
   print 'INDEX=%s MESSAGE=%s' % (numid, msgid)
   
   # 6. copy message file
   repid = str(int(msgid) / 1000)
   fmsg = os.path.join(run['tmp_user'], 'msg%s' % msgid)
   txtmsg = convert(d['TEXTE'], std_encoding, db_encoding)
   open(fmsg, 'w').write(txtmsg + '\n')
   cmd = []
   cmd.append(os.path.join(run['rep_agla'], 'roundup_cp_uid'))
   cmd.append('put')
   cmd.append(fmsg)
   cmd.append('%s' % repid)
   iret, output = run.Shell(' '.join(cmd))
   if iret != 0:
      run.Mess(_('Error message: %s') % output)
      run.Mess(_('Error occurs during copying message file'), _('<F>_COPY'))
   
   # 7. study files
   if fichetude == 1:
      cmd = []
      cmd.append(os.path.join(run.astk_serv_root, 'bin', 'as_rex_prof'))
      cmd.append(fprof)
      cmd.append('%06d' % numid)
      iret, output = run.Shell(' '.join(cmd))
      if iret != 0:
         run.Mess(_('Error occurs during copying study files'), _('<F>_COPY'))

#-------------------------------------------------------------------------------
def Histor(run, *args):
   """Extract the content of some issues from REX database.
   """
   if len(args) != 2:
      run.parser.error(_("'--%s' requires two arguments") % run.current_action)

   iret = 0
   on_machref = run.get('rep_agla', 'local') != 'local'
   if not on_machref:
      run.Mess(_('Only available on the AGLA machine'), _('<F>_AGLA_ERROR'))
   
   # 0. check imports
   if not imports_succeed:
      run.Mess(_('Imports of REX interface failed.'), _('<F>_IMPORT_ERROR'))
   
   # 1. copy input file
   jn = run['num_job']
   ffich = os.path.join(run['tmp_user'], 'hist_input.'+jn)
   kret = run.Copy(ffich, args[0], niverr=_('<F>_COPYFILE'))
   
   # 2. read input file
   hist_content = open(ffich, 'r').read()
   expr = re.compile('([0-9]+)', re.MULTILINE)
   l_nf = [int(n) for n in expr.findall(hist_content)]
   
   # 3. open database connection
   try:
      c = db.CONNECT('REX',
            rcdir = os.path.join(run.astk_serv_root, 'conf'),
            verbose=run['debug'])
   except Exception, msg:
      run.Mess(msg, _('<F>_DB_ERROR'))
   
   # 4. read issues from database
   prec_user = None
   histor = InitHistor(format=run['format'], url=run['rex_url'])
   for numf in l_nf:
      # 4.1. read the issue
      issue = ISSUE({'id' : numf}, c)
      try:
         issue.read()
      except ReadDBError:
         run.Mess(_('Unable to read issue %s') % numf, _('<E>_UNKNOWN_ISSUE'))
         continue
      
      # 4.2.
      # 4.2.1. check status
      status = issue['_status'].GetPrimValue()
      if status != run['status'] and run['status'] != 'all':
         run.Mess(_("Status of issue %s is not '%s' (%s)") \
               % (numf, run['status'], status), _('<E>_UNEXPECTED_VALUE'))
      
      # 4.2.2. alarm on impactDoc
      if issue['_impactDoc'] != None:
         run.Mess(_('modified documents : %s') % issue['_impactDoc'], \
               _('<A>_DOCUMENTATION'))
      
      # 4.3.1. user header
      if numf == l_nf[0] or prec_user == None:
         prec_user = issue['_assignedto']
         histor.BeginUser(prec_user)
      
      if issue['_assignedto'] != None and prec_user != None \
            and issue['_assignedto']['id'] != prec_user['id']:
         if prec_user != None:
            histor.EndUser(prec_user)
         prec_user = issue['_assignedto']
         histor.BeginUser(prec_user)
      
      # 4.3.2. get message file of answer
      l_rep = []
      d_msg = issue.GetLinks()
      # allow to print all messages
      if run['all_msg']:
         l_msgid = d_msg.keys()
         l_msgid.sort()
         l_msgid.reverse()
      else:
         l_msgid = [max(d_msg.keys())]
      for i, msgid in enumerate(l_msgid):
         repid = str(int(msgid) / 1000)
         msgfile = '%s%smsg%s' % (repid, os.sep, msgid)
         fmsg = os.path.join(run['rep_tmp'], 'msg%s' % msgid)
         cmd = []
         cmd.append(os.path.join(run['rep_agla'], 'roundup_cp_uid'))
         cmd.append('get')
         cmd.append('%s%smsg%s' % (repid, os.sep, msgid))
         cmd.append(run['rep_tmp'])
         iret, output = run.Shell(' '.join(cmd))
         if iret != 0 or not os.path.exists(fmsg):
            run.Mess(_('Error message: %s') % output)
            run.Mess(_('Error occurs during copying message file %s, ' \
                       'only the summary will be printed') % msgid, _('<A>_COPY'))
         else:
            txt = open(fmsg, 'r').read()
            if len(l_msgid) > 1:
               l_rep.append(_('message %d :') % (len(l_msgid) - i))
            txtmsg = convert(txt, db_encoding, std_encoding)
            l_rep.append(txtmsg)
      
      # 4.3.3. add this issue to the histor object
      histor.AddIssue(issue, l_rep)
      
      # 4.3.4. last issue
      if numf == l_nf[-1] and prec_user != None:
         histor.EndUser(prec_user)
   
   # 5. copy histor file
   ffich = os.path.join(run['tmp_user'], 'hist_ouput.'+jn)
   open(ffich, 'w').write(repr(histor))
   kret = run.Copy(args[1], ffich, niverr=_('<F>_COPYFILE'))

#-------------------------------------------------------------------------------
def Solder(run, *args):
   """Fill corrVdev/corrVexpl fields and close issues found in a histor file.
   """
   if len(args) != 1:
      run.parser.error(_("'--%s' requires one argument") % run.current_action)
   
   on_machref = run.get('rep_agla', 'local') != 'local'
   if not on_machref:
      run.Mess(_('Only available on the AGLA machine'), _('<F>_AGLA_ERROR'))
   try:
      is_adm = grp.getgrgid(os.getgid())[0] == run.get('agla_adm_group')
      if not is_adm:
         raise KeyError
   except KeyError, msg:
      run.Mess(_('insufficient privileges to close issues !'), _('<F>_AGLA_ERROR'))
   
   # 0. check imports
   if not imports_succeed:
      run.Mess(_('Imports of REX interface failed.'), _('<F>_IMPORT_ERROR'))
   
   # 1. read histor file
   jn = run['num_job']
   ffich = os.path.join(run['tmp_user'], 'hist_input.'+jn)
   kret = run.Copy(ffich, args[0], niverr=_('<F>_COPYFILE'))
   
   hist_content = open(ffich, 'r').read()
   expr = re.compile('RESTITUTION FICHE +([0-9]+)', re.MULTILINE)
   l_nf = [int(n) for n in expr.findall(hist_content)]
   
   # 2. get version number and database fields to fill
   REPREF = os.path.join(run.aster_root, run['version_dev'])
   run.Mess(_('Close issues for :'))
   iret, l_res = as_maintenance.GetVersion(run, silent=False, REPREF=REPREF)
   tagv = '.'.join(l_res[:3])
   if l_res[4]:
      typv = 'expl'
      autr = 'dev'
   else:
      typv = 'dev'
      autr = 'expl'
   d_champ = {
      'a_corrige' : '_corrV' + typv,
      'v_correct' : '_verCorrV' + typv,
      'a_tester'  : '_corrV' + autr,
      'v_tester'  : '_verCorrV' + autr,
   }
   
   # 3. open database connection
   try:
      c = db.CONNECT('REX',
            rcdir = os.path.join(run.astk_serv_root, 'conf'),
            verbose=run['debug'])
   except Exception, msg:
      run.Mess(msg, _('<F>_DB_ERROR'))
   
   try:
      etat = ReadDB(STATUS, c)
      prod = ReadDB(PRODUIT, c)
   except ReadDBError:
      run.Mess(_('Unable to read status and products'), _('<F>_DB_ERROR'))
   
   # 4. read issues from database
   req_status = 'valide_EDA'
   for numf in l_nf:
      # 4.1. read the issue
      try:
         issue = ISSUE(numf, c)
      except ReadDBError:
         run.Mess(_('Unable to read issue %s') % numf, _('<E>_UNKNOWN_ISSUE'))
         continue
      
      # 4.2. check issue values
      status = issue['_status'].GetPrimValue()
      if status != req_status:
         run.Mess(_("Status of issue %s is not '%s' (%s)") \
               % (numf, req_status, status), _('<A>_UNEXPECTED_VALUE'))
         issue['_status'] = etat[req_status]
      
      if issue['_produit'].GetPrimValue() == 'Code_Aster':
         if not issue[d_champ['a_corrige']] \
            and issue['_type'].GetPrimValue() != 'aide utilisation':
            issue[d_champ['a_corrige']] = YES
            run.Mess(_("issue %s should not been solved in V%s") \
                  % (numf, typv), _('<A>_UNEXPECTED_VALUE'))
      else:
         issue[d_champ['a_corrige']] = NO
         issue[d_champ['a_tester']] = NO
                  
      # 4.3. fill issue fields
      issue[d_champ['v_correct']] = tagv
      
      # 4.4. close issue ?
      if not issue[d_champ['a_tester']] or not issue[d_champ['v_tester']] in ('', None):
         issue['_status'] = etat['ferme']
         run.Mess(_('issue %s is closed') % numf)
      else:
         run.Mess(_('issue %s must be solved in V%s too') % (numf, autr))
      
      # 4.5. write issue in database
      try:
         if not run['debug']:
            issue.write(force=True)
         else:
            issue.repr()
      except WriteDBError, msg:
         run.Mess(_('error occurs during writing issue'), _('<F>_DB_ERROR'))






# -*- coding: iso-8859-1 -*-

# ==============================================================================
# COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
# ==============================================================================

"""Definition of ASTER_SYSTEM class.
"""

__revision__ = "$Id: as_system.py 2657 2006-12-19 15:55:44Z courtois $"


# ----- differ messages translation
def _(mesg):
   return mesg

import os
import sys
import re
import popen2
import shutil
import glob
import getpass
import resource
from types import StringType, ListType, TupleType, IntType, LongType, \
                  InstanceType
EnumTypes = (ListType, TupleType)

# colors and tty...
label = { 'OK' : _('  OK  '), 'FAILED' : _('FAILED'), 'SKIP' : _(' SKIP ') }
if sys.stdout.isatty():
   label['OK']     = '\033[1;32m%s\033[m\017' % label['OK']
   label['FAILED'] = '\033[1;31m%s\033[m\017' % label['FAILED']
#   label['SKIP']   = '\033[1;34m%s\033[m\017' % label['SKIP']
for k in label:
   label[k] = '['+label[k]+']'

#-------------------------------------------------------------------------------
def _exitcode(status, default=0):
   """Extrait le code retour du status. Retourne `default` si le process
   n'a pas fini pas exit.
   """
   iret = default
   if os.WIFEXITED(status):
      iret = os.WEXITSTATUS(status)
   return iret

#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
class ASTER_SYSTEM:
   """Class to encapsultate "system" commands on local or remote machine.
   Available methods :
      'Copy', 'Shell', 'IsRemote', 'MkDir', 'Delete', 'Exists', 'IsDir',
      'Gzip', 'Gunzip', 'AddToEnv', 'GetHostName', 'PathOnly'
      'VerbStart, 'VerbEnd', 'VerbIgnore', 'SetLimit', 'Which'
   Supported protocols :
   - for copy :
      LOCAL, RCP, SCP, RSYNC, HTTP
   - for shell execution :
      LOCAL, RSH, SSH
   """
   # this value must be set during installation step.
   MaxCmdLen = 1024
   # line length -9
   _LineLen = 80-9
   # required parameters for initialisation
   RequiredParam = ['remote_copy_protocol', 'remote_shell_protocol']
#-------------------------------------------------------------------------------
   def __init__(self, run, **kargs):
      """run : ASTER_RUN object
      OR just a dictionnary to define 'RequiredParam' + 'verbose', 'debug'
      """
      self.verbose = run['verbose']
      self.debug = run['debug']
      if kargs.has_key('maxcmdlen'):
         self.MaxCmdLen = kargs['maxcmdlen']
      # fill param
      self.param = {}
      for key in self.RequiredParam:
         self.param[key] = run[key]
      # ----- reference to ASTER_RUN object which manages the execution
      self.run = run
      # informations
      self.info =  {
         # store local user (will be the default for remote machines)
         'local_user' : getpass.getuser(),
         'local_host' : os.uname()[1],
      }
      # check if protocols are supported
      if not self.param['remote_shell_protocol'] in ('LOCAL', 'RSH', 'SSH'):
         self._mess(_('unknown protocol : %s') % self.param['remote_shell_protocol'],
               _('<A>_ALARM'))
      if not self.param['remote_copy_protocol'] in ('LOCAL', 'RCP', 'SCP', 'RSYNC', 'HTTP'):
         self._mess(_('unknown protocol : %s') % self.param['remote_copy_protocol'],
               _('<A>_ALARM'))
      self._dbg('ASTER_SYSTEM param :', self.param)
      # add shortcuts - example: run.Copy instead of run.system.Copy
      if type(run) is InstanceType:
         for meth in ASTER_SYSTEM.__dict__.keys():
            if meth[0] == meth[0].upper() and not hasattr(run, meth) \
                  and hasattr(self, meth) and callable(getattr(self, meth)):
               setattr(run, meth, getattr(self, meth))

#-------------------------------------------------------------------------------
   def _mess(self, msg, cod='', store=False):
      """Just print a message
      """
      if hasattr(self.run, 'Mess'):
         self.run.Mess(msg, cod, store)
      else:
         print '%-18s %s' % (cod, msg)

#-------------------------------------------------------------------------------
   def _dbg(self, *args, **kargs):
      """Print debug informations
      """
      if hasattr(self.run, 'DBG'):
         self.run.DBG(*args, **kargs)
      elif self.debug:
         print '<DBG> %s // %s' % (str(args), str(kargs))

#-------------------------------------------------------------------------------
   def VerbStart(self, cmd, verbose=None):
      """Start message in verbose mode
      """
      Lm = self._LineLen
      if verbose == None:
         verbose = self.verbose
      if verbose:
         pcmd = cmd
         if len(cmd) > Lm-2 or cmd.count(os.linesep) > 0:
            pcmd = pcmd+os.linesep+' '*Lm
         print ('%-'+str(Lm)+'s') % (pcmd, ),
         sys.stdout.flush()

#-------------------------------------------------------------------------------
   def VerbEnd(self, iret, output='', verbose=None):
      """Ends message in verbose mode
      """
      if verbose == None:
         verbose = self.verbose
      if verbose:
         if iret == 0:
            print label['OK']
         else:
            print label['FAILED']
            print _('Exit code : %d') % iret
         if (iret != 0 or self.debug) and output:
            print output

#-------------------------------------------------------------------------------
   def VerbIgnore(self, verbose=None):
      """Ends message in verbose mode
      """
      if verbose == None:
         verbose = self.verbose
      if verbose:
         print label['SKIP']

#-------------------------------------------------------------------------------
   def local_shell(self, cmd, bg=False, verbose=False, follow_output=False,
         alt_comment=None, interact=False, capturestderr=True):
      """Execute a command shell
         cmd      : command
         bg       : put command in background if True
         verbose  : print status messages during execution if True
         follow_output : follow interactively output of command
         alt_comment : print this "alternative comment" instead of "cmd"
         interact : allow the user to interact with the process
            (don't close stdin). bg=True implies interact=False.
      Return :
         iret     : exit code if bg = False,
                    process id if bg = True
         output   : output lines (as string)
      """
      if not alt_comment:
         alt_comment = cmd
      if bg:
         interact = False
      if len(cmd)>self.MaxCmdLen:
         self._mess((_('length of command shell greater '\
               'than %d characters.') % self.MaxCmdLen), _('<A>_ALARM'))
      self._dbg('cmd :', cmd, 'background : %s' % bg)
      # exec
      self.VerbStart(alt_comment, verbose=verbose)
      if follow_output and verbose:
         print os.linesep+_('Command output :')
#MC A REVOIR : il faudrait connecter tochild à stdin et fromchild à stdout
# faut-il le faire dans un thread séparé....? plus tard ;-)
      # run interactive command
      if interact:
         iret = os.system(cmd)
         return _exitcode(iret), ''
      # use popen to manipulate stdout/stderr
      output = []
      if capturestderr:
         p = popen2.Popen4(cmd)
      else:
         p = popen2.Popen3(cmd, capturestderr=False)
      p.tochild.close()
      if not bg:
         if not follow_output:
            output = p.fromchild.readlines()
         else:
            while p.poll() == -1:
               stmp = p.fromchild.readline()
               if stmp != '':
                  # `os.linesep` already here...
                  print stmp,
                  output.append(stmp)
               sys.stdout.flush()
            # to be sure to empty the buffer
            end = p.fromchild.readlines()
            print ''.join(end)
            output.extend(end)
         try:
            iret = _exitcode(p.wait())
         except OSError, msg:
            self._dbg("OSError = %s" % str(msg))
            iret = 4
      else:
         iret = 0
      p.fromchild.close()
      output = ''.join(output)

      # repeat header message
      if follow_output:
         self.VerbStart(alt_comment, verbose=verbose)
      mat = re.search('EXIT_CODE=([0-9]+)', output)
      if mat != None:
         try:
            iret = int(mat.group(1))
         except ValueError:
            pass
      self._dbg("iret = %d\noutput = %s" % (iret, output))
      self.VerbEnd(iret, output, verbose=verbose)
      if bg:
         iret = p.pid
         if verbose:
            print 'Process ID : ', iret
      return iret, output

#-------------------------------------------------------------------------------
   def filename2dict(self, s):
      """Convert file name 's' given in format [[user[:passwd]@]machine:]name
      into a dictionnary with keys = user, passwd, mach, name
      """
      d = {
         'user'   : '',
         'passwd' : '',
         'mach'   : '',
         'name'   : ''
      }
      if not type(s) is StringType:
         self._mess(_('unexpected type'), _('<F>_PROGRAM_ERROR'))
      mat = re.search('(.*):(.*)@(.*):(.*)', s)
      n = 4
      if mat == None:
         mat = re.search('(.*)@(.*):(.*)', s)
         n = 3
         if mat == None:
            mat = re.search('(.*):(.*)', s)
            n = 2
            if mat == None:
               d['name'] = s
               n = 1
      if n >= 2:
         g = list(mat.groups())
         d['name'] = g.pop()
         d['mach'] = g.pop()
      if n >= 3:
         d['user'] = g.pop()
      if n >= 4:
         d['passwd'] = d['user']
         d['user'] = g.pop()
      if d['mach'] != '' and d['user'] == '':
         d['user'] = self.info['local_user']
      return d

#-------------------------------------------------------------------------------
   def PathOnly(self, filename):
      """Return only the path of file given as [[user[:passwd]@]machine:]name.
      Very useful when `filename` contains 'user@mach:' string but refers to a
      local filename.
      """
      return self.filename2dict(filename)['name']

#-------------------------------------------------------------------------------
   def Copy(self, dest, *src, **opts):
      """Copy files/directories (from 'src' list) to 'dest'
      Remote names format :
         [[user[:passwd]@]machine:]name
      where name is an absolute or a relative path.
      optionnal arguments : verbose, follow_output passed to local_shell,
         niverr : default is set to <F>_COPY_ERROR, but may be '<A>' or less !
         protocol : to use a different protocol just for this copy.
      Note : if src[i] is a directory and dest doesn't exist
             dest is created and the content of src[i] is copied into dest.
             This is not a default behavior of rsync which would create
             `basename src[i]` in dest even if dest doesn't exist.
      """
      # cp command
      if sys.platform in ('linux2', 'x86_64', 'cygwin'):
         cpcmd = 'cp -L -r'
      else:
         cpcmd = 'cp -r'
      
      iret = 0
      kargs = {}
      # take values from opts if exist
      kargs['verbose'] = opts.get('verbose', self.verbose)
      kargs['follow_output'] = opts.get('follow_output', False)
      if opts.has_key('niverr'):
         niverr = opts['niverr']
      else:
         niverr = _('<F>_COPY_ERROR')
      ddest = self.filename2dict(dest)
      if self.IsRemote(dest) \
            or (ddest['user'] != '' and ddest['user'] != self.info['local_user']):
         fdest = ddest['user']+'@'+ddest['mach']+':'+ddest['name']
      else:
         fdest = ddest['name']
      self._dbg('source list : %s' % (src,), 'destination : %s' % fdest)

      if len(src) < 1:
         self._mess(_('no source file to copy'), _('<A>_ALARM'))

      for f in src:
         # here because we can change 'proto' if necessary
         proto = opts.get('protocol', self.param['remote_copy_protocol'])
         jret = 0
         df = self.filename2dict(f)
         if self.IsRemote(f):
            fsrc = df['user']+'@'+df['mach']+':'+df['name']
         else:
            fsrc = df['name']
         cmd = ''
         tail = '.../'+'/'.join(f.split('/')[-2:])
         kargs['alt_comment'] = 'copying %s' % tail
         if df['mach'] == '' and ddest['mach'] == '':
            cmd = cpcmd+' '+fsrc+' '+fdest
         else:
            if proto == 'RSYNC' and df['mach'] != '' and ddest['mach'] != '':
               proto = 'RCP'
               self._mess(_("copying a remote file to another remote server " \
                     "isn't allowed through RSYNC, trying with RCP."))
            if proto == 'RCP':
               cmd = 'rcp -r '+fsrc+' '+fdest
            elif proto == 'SCP':
               cmd = 'scp -rBCq '+fsrc+' '+fdest
            elif proto == 'RSYNC':
               if self.IsDir(f) and not self.Exists(dest):
                  self.MkDir(dest)
                  cmd = 'rsync -rz '+os.path.join(fsrc, '*')+' '+fdest
               else:
                  cmd = 'rsync -rz '+fsrc+' '+fdest
            elif proto == 'HTTP':
               str_user = ''
               if not df['user'] in ('', 'anonymous'):
                  str_user = df['user']+'@'
               # dest must be local
               if ddest['mach'] == '':
                  cmd = 'wget http://'+str_user+df['mach']+df['name']+' -O '+fdest
               else:
                  cmd = ''
                  self._mess(_('remote destination not allowed through %s' \
                        ' : %s') % (proto, fdest), niverr)
         if cmd != '':
            jret, out = self.local_shell(cmd, **kargs)
            if jret != 0 and niverr != 'SILENT':
               self._mess(_('error during copying %s to %s') % (f, fdest) \
                  + os.linesep + _('message : %s') % out, niverr)
         else:
            self._mess(_('unexpected error or unknown protocol : %s') \
                   % proto, niverr)
         iret = max(jret, iret)
      return iret

#-------------------------------------------------------------------------------
   def Shell(self, cmd, mach='', user='', **opts):
      """Execute a command shell on local or remote machine.
      Options : see local_shell
      """
      iret = 1
      # valeurs par défaut
      kargs = {
         'verbose'         : self.verbose,
         'bg'              : False,
         'interact'        : False,
         'follow_output'   : False,
         'alt_comment'     : cmd,
         'capturestderr'   : True,
      }
      # surcharge par opts
      kargs.update(opts)

      proto = self.param['remote_shell_protocol']
      #if cmd.find('"') != -1:
      #      self._mess(_("""an error may occurs because of '"' character ' \
      #            'in command"""), _('<A>_ALARM'))
      #cmd = "sh -c '"+cmd+"'"
      if mach == '':
         iret, out = self.local_shell(cmd, **kargs)
      else:
         if user == '':
            user = getpass.getuser()
         action = ''
         if proto == 'RSH':
            action = 'rsh -n -l '+user+' '+mach+' '+cmd
         elif proto == 'SSH':
            action = 'ssh -n -l '+user+' '+mach+' '+cmd
         if action != '':
            iret, out = self.local_shell(action, **kargs)
         else:
            self._mess(_('unknown protocol : %s') % proto, _('<A>_ALARM'))
      return iret, out

#-------------------------------------------------------------------------------
   def Gzip(self, src, **opts):
      """Compress file or content of directory 'src'.
      optionnal arguments : verbose, niverr, cmd (to override gzip command).
      Only local files/directories supported !
      """
      return self._gzip_gunzip('gzip', src, **opts)

#-------------------------------------------------------------------------------
   def Gunzip(self, src, **opts):
      """Decompress file or content of directory 'src'.
      optionnal arguments : verbose, niverr, cmd (to override gzip -d command).
      Only local files/directories supported !
      """
      return self._gzip_gunzip('gunzip', src, **opts)

#-------------------------------------------------------------------------------
   def _gzip_gunzip(self, mode, src, **opts):
      """Called by Gzip (mode=gzip) and Gunzip (mode=gunzip) methods.
      Return status and filename of src after de/compression.
      Only local files/directories supported !
      """
      para = {
         'gzip' : {
            'cmd'     : 'gzip ',
            'niverr'  : _('<F>_COMPRES_ERROR'),
            'msg0'    : _('no file/directory to compress'),
            'comment' : _('compressing %s'),
            'msgerr'  : _('error during compressing %s'),
         },
         'gunzip' : {
            # sometimes gunzip doesn't exist, gzip seems safer
            'cmd'     : 'gzip -d ',
            'niverr'  : _('<F>_DECOMPRESSION'),
            'msg0'    : _('no file/directory to decompress'),
            'comment' : _('decompressing %s'),
            'msgerr'  : _('error during decompressing %s'),
         },
      }
      if not mode in para.keys():
         self._mess(_('unknown mode : %s') % mode, _('<F>_PROGRAM_ERROR'))
      iret = 0
      if opts.has_key('cmd'):
         cmd = opts['cmd']
      else:
         cmd = para[mode]['cmd']
      if opts.has_key('verbose'):
         verbose = opts['verbose']
      else:
         verbose = self.verbose
      if opts.has_key('niverr'):
         niverr = opts['niverr']
      else:
         niverr = para[mode]['niverr']

      if len(src) < 1:
         self._mess(para[mode]['msg0'], _('<A>_ALARM'))
         return iret

      if not os.path.isdir(src):
         if mode == 'gzip':
            name = src+'.gz'
         else:
            name = re.sub('\.gz$', '', src)
         lf = [src]
      else:
         name = src
         lf = glob.glob(os.path.join(src, '*'))
      for f in lf:
         jret = 0
         comment = para[mode]['comment'] % f
         if os.path.isdir(f):
            self.VerbStart(comment, verbose=verbose)
            self.VerbIgnore(verbose=verbose)
         else:
            jret, out = self.local_shell(cmd+f, verbose=verbose, alt_comment=comment)
         if jret != 0:
            self._mess(para[mode]['msgerr'] % f, niverr)
         iret = max(iret, jret)
      return iret, name

#-------------------------------------------------------------------------------
   def IsRemote(self, path):
      """Return True if 'path' seems to be on a remote host.
      NB : we suppose that host and host.domain are identical.
      """
      dico  = self.filename2dict(path)
      mach = dico['mach'].split('.')[0]
      diff_host = ( mach != '' and mach != self.info['local_host'])
      return diff_host

#-------------------------------------------------------------------------------
   def Exists(self, path):
      """Return True if 'path' exists (file or directory).
      """
      iret = 0
      exists = True
      dico = self.filename2dict(path)
      if self.IsRemote(path):
         cmd = "\"sh -c 'if test -f "+dico['name']+' ; then echo FILE_EXISTS; ' \
                         "elif test -d "+dico['name']+' ; then echo DIR_EXISTS ; ' \
                                                         "else echo FALSE ; fi'\""
         iret, out = self.Shell(cmd, dico['mach'], dico['user'], verbose=self.verbose)
         if out.find('FALSE') > -1:
            exists = False
      else:
         exists = os.path.exists(dico['name'])
      return exists

#-------------------------------------------------------------------------------
   def IsDir(self, path):
      """Return True if 'path' is a directory.
      """
      iret = 0
      isdir = False
      dico = self.filename2dict(path)
      if self.IsRemote(path):
         cmd = "\"sh -c 'if test -d "+dico['name']+' ; then echo IS_DIRECTORY; ' \
                                             "else echo FALSE ; fi'\""
         iret, out = self.Shell(cmd, dico['mach'], dico['user'], verbose=self.verbose)
         if out.find('IS_DIRECTORY') > -1:
            isdir = True
      else:
         isdir = os.path.isdir(dico['name'])
      return isdir

#-------------------------------------------------------------------------------
   def IsText(self, path):
      """Return True if `path` is a text file.
      Warning : return False if `path` is remote
      """
      iret = 0
      istext = False
      dico = self.filename2dict(path)
      if not self.IsRemote(path):
         cmd = "file %s" % path
         iret, out = self.Shell(cmd, dico['mach'], dico['user'], verbose=self.verbose)
         if out.lower().find('text') > -1:
            istext = True
      return istext

#-------------------------------------------------------------------------------
   def MkDir(self, rep, niverr=None, verbose=None):
      """Create the directory 'rep' (mkdir -p ...)
      """
      if niverr == None:
         niverr = _('<F>_MKDIR_ERROR')
      if verbose == None:
         verbose = self.verbose
      iret = 0
      dico = self.filename2dict(rep)
      if self.IsRemote(rep):
         cmd = 'mkdir -p '+dico['name']
         iret, out = self.Shell(cmd, dico['mach'], dico['user'], verbose=verbose)
      else:
         self.VerbStart(_('creating directory %s') % dico['name'], verbose)
         # ----- it's not tested in remote
         if self.IsDir(rep):
            iret = 0
            self.VerbIgnore(verbose)
         else:
            s = ''
            try:
               os.makedirs(dico['name'])
            except OSError, s:
               iret = 4
            self.VerbEnd(iret, s, verbose)
      if iret != 0:
         self._mess(_('can not create directory %s') % rep, niverr)
      return iret

#-------------------------------------------------------------------------------
   def Delete(self, rep, remove_dirs=True, verbose=None):
      """Delete a file or a directory tree (rm -rf ...).
      Set 'remove_dirs' to False to be sure to delete 'rep' only if it's a file.
      """
      if verbose == None:
         verbose = self.verbose
      iret = 0
      dico = self.filename2dict(rep)
      # preventing to delete first level directories (as /, /home, /usr...)
      if dico['name'][0] == '/' and len(dico['name'][:-1].split(os.sep)) <= 2:
         self._mess(_('deleting this directory seems too dangerous. ' \
               '%s has not been removed') % dico['name'], _('<A>_ALARM'))
         return 1
      
      if remove_dirs:
         cmd = 'rm -rf '+dico['name']
      else:
         cmd = 'rm -f '+dico['name']
      
      comment = _('deleting %s') % rep
      if self.IsRemote(rep):
         iret, out = self.Shell(cmd, dico['mach'], dico['user'],
                                alt_comment=comment, verbose=verbose)
      else:
         iret, out = self.Shell(cmd, alt_comment=comment, verbose=verbose)
      return

#-------------------------------------------------------------------------------
   def AddToEnv(self, profile):
      """Read 'profile' file (with sh/bash/ksh syntax) and add updated
      variables to os.environ.
      """
      def env2dict(s):
         """Convert output to a dictionnary."""
         l = s.split(os.linesep)
         d = {}
         for line in l:
            mat = re.search('^([-a-zA-Z_0-9@\+]+)=(.*$)', line)
            if mat != None:
               d[mat.group(1)] = mat.group(2)
         return d
      if not os.path.isfile(profile):
         self._mess(_('file not found : %s') % profile, '<A>_FILE_NOT_FOUND')
         return
      # read initial environment
      iret, out = self.Shell('sh -c env')
      env_init = env2dict(out)
      if iret != 0:
         self._mess(_('error getting environment'), '<E>_ABNORMAL_ABORT')
         return
      # read profile and dump modified environment
      iret, out = self.Shell('sh -c ". %s ; env"' % profile)
      env_prof = env2dict(out)
      if iret != 0:
         self._mess(_('error reading profile : %s') % profile,
               '<E>_ABNORMAL_ABORT')
         return
      # "diff"
      for k, v in env_prof.items():
         if env_init.get(k, None) != v:
            if self.debug:
               print 'AddToEnv adding : %s=%s' % (k, v)
            os.environ[k] = v

#-------------------------------------------------------------------------------
   def GetHostName(self, host=None):
      """Return hostname of the machine 'host' or current machine if None.
      """
      from socket import gethostname, gethostbyaddr
      if host == None:
         host = gethostname()
      fqn, alias, ip = gethostbyaddr(host)
      if fqn.find('localhost')>-1:
         alias = [a for a in alias if a.find('localhost')<0]
         if len(alias)>0:
            fqn = alias[0]
         for a in alias:
            if a.find('.')>-1:
               fqn = a
               break
      return fqn

#-------------------------------------------------------------------------------
   def SendMail(self, dest, author=None, subject='no subject', text=''):
      """Send a message by mail.
      Use ", " in `dest` to separate multiple recipients.
      """
      sign = 'sent by as_run/SendMail from %s' % self.GetHostName()
      sign = os.linesep*2 + '-'*len(sign) + os.linesep \
           + sign \
           + os.linesep   + '-'*len(sign) + os.linesep
      try:
         import smtplib
         from email.MIMEText import MIMEText
      except ImportError:
         self._mess(_('Can not send mail from this machine'),
            _('<A>_NO_MAILER'))
         return
      dest = [s.strip() for s in dest.split(',')]
      msg = MIMEText(text+sign)
      msg['Subject'] = subject
      if author == None:
         author = '%s@%s' % (getpass.getuser(), self.GetHostName())
      msg['From'] = author
      msg['To'] = ', '.join(dest)
      s = smtplib.SMTP()
      s.connect()
      s.sendmail(msg['From'], dest, msg.as_string())
      s.close()

#-------------------------------------------------------------------------------
   def SetLimit(self, what, *l_limit):
      """Set a system limit.
      `what` is one of CORE, CPU... (see help of resource module).
      If provided `l_limit` contains (soft limit, hard limit).
      """
      nomp = 'RLIMIT_%s' % what.upper()
      param = getattr(resource, nomp)
      if param != None:
         if len(l_limit) == 1:
            l_limit = (l_limit[0], l_limit[0])
         elif len(l_limit) != 2:
            l_limit = (-1, -1)
         l_limit = list(l_limit)
         for i, lim in enumerate(l_limit):
            if type(lim) not in (IntType, LongType):
               l_limit[i] = -1
         try:
            self._dbg([what, nomp, l_limit])
            resource.setrlimit(param, l_limit)
         except Exception, msg:
            self._mess(_('unable to set %s limit to %s') % (nomp, l_limit))

#-------------------------------------------------------------------------------
   def Which(self, cmd):
      """Same as `which cmd`. Returns the path found or None.
      """
      ldirs = os.environ.get('PATH').split(':')
      for d in ldirs:
         path = os.path.join(d, cmd)
         if os.path.isfile(path) and os.access(path, os.X_OK):
            return path
      return None

#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
class ASTER_SYSTEM_MINIMAL(ASTER_SYSTEM):
   """Fake ASTER_RUN to use easily ASTER_SYSTEM outside of as_run.
   """
#-------------------------------------------------------------------------------
   def __init__(self, **kargs):
      """run : ASTER_RUN object
      OR just a dictionnary to define 'RequiredParam' + 'verbose', 'debug'
      """
      opts = { 
         'debug'   : False,
         'verbose' : False,
         'remote_copy_protocol' : 'SCP',
         'remote_shell_protocol' : 'SSH',
      }
      opts.update(kargs)
      ASTER_SYSTEM.__init__(self, run=opts)
